@@ -121,7 +121,7 @@ void DocAction::Undo()
 
 void DocAction::DoTextInsert()
 {
-	InsertNodeText(m_nNodeIndex, m_nTextStartPos, m_strNodeText.c_str());
+	InsertNodeText(m_nNodeIndex, m_nTextStartPos, m_strNodeText.c_str(), &m_lstPictures);
 	if(m_bSelected){
 		g_text.SelectRange(m_nTextStartPos, m_nTextStartPos+m_strNodeText.size());
 
