@@ -86,6 +86,9 @@ public:
 	int m_nTextStartPos;		//text offset inside the node
 	bool m_bSelected;		//was text in case selected
 
+	//additional data for node text deleting
+	std::vector<PixInfo> m_lstPictures;
+
 	// node renaming/inserting/deleting
 	int m_nNodeIndex;			//node position (placement)
 	int m_nNodeSibling;			//sibling index (index in the )
