@@ -29,7 +29,7 @@ void FindDialog::Create()
 	//fill target combo
 	GtkWidget *cbo3	= lookup_widget(m_pDialog, "comboboxentry3");
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cbo3), _("Node title/contents"));
-	gtk_combo_box_append_text(GTK_COMBO_BOX(cbo3), _("Node keywords"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(cbo3), _("Node tags"));
 	gtk_combo_box_set_active(GTK_COMBO_BOX(cbo3), 0);
 }
 
