@@ -32,6 +32,9 @@ public:
 	GtkWidget *m_pToolbarColor;
 	GtkWidget *m_pToolbarAttachment;
 	GtkWidget *m_pToolbarStrike;
+	GtkWidget *m_pToolbarBkgColor;
+	GtkWidget *m_pToolbarUndo;
+	GtkWidget *m_pToolbarRedo;
 
 protected:
 	GtkWidget* create_main_win ();
