@@ -10,24 +10,35 @@
 #include "support.h"
 #include "callbacks.h"
 #include "gui/FileDialog.h"
+#include "lib/IniFile.h"
+#include "lib/FilePath.h"
 #include "lib/debug.h"
 #include "config.h"
+#include <gdk/gdkkeysyms.h>
 
 #ifdef _WIN32
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
+ #include <io.h> //access
+ #define access _access
 #endif
 
 extern GtkWidget *g_pNodePropWidget;
 extern GtkWidget *window1;
+extern IniFile g_objIni;
+static GtkWidget *g_cancelbtn = NULL;
+static GtkWidget *g_editor = NULL;
+extern bool g_bCloseDlgOnTextEnter;
 
 const char *InternalIcon_Index2Name(int nIndex);
 const char **InternalIcon_GetFromIdx(int);
 int InternalIcon_Name2Index(const char *);
 int gtkMessageBox(const char *szText, int nButtons = GTK_BUTTONS_OK, int nIcon = GTK_MESSAGE_INFO);
 const char *combo_get_text(GtkWidget *cbo3);
-
+static gint entry_keyboard_handler(GtkWidget *widget, GdkEventKey *event);
 static void on_icon_none_clicked (GtkMenuItem *menuitem, gpointer user_data);
 static void on_icon_internal_clicked (GtkMenuItem *menuitem, gpointer user_data);
 static void on_icon_custom_clicked (GtkMenuItem *menuitem, gpointer user_data);
@@ -59,7 +70,9 @@ void NodePropertiesDlg::Create()
 	if(m_strTitle.c_str())
 	{
 		GtkWidget *entry = lookup_widget(m_pDialog, "entry_node_title");
-		gtk_entry_set_text(GTK_ENTRY(entry), m_strTitle.c_str());
+		//gtk_entry_set_text(GTK_ENTRY(entry), m_strTitle.c_str());
+		GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(entry));
+		gtk_text_buffer_set_text(buffer, m_strTitle.c_str(), -1);
 	}
 
 	//fill internal icons combo
@@ -104,7 +117,12 @@ void NodePropertiesDlg::Create()
 const char *NodePropertiesDlg::GetNodeTitle()
 {
 	GtkWidget *entry = lookup_widget(m_pDialog, "entry_node_title");
-	return gtk_entry_get_text(GTK_ENTRY(entry));
+	//return gtk_entry_get_text(GTK_ENTRY(entry));
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(entry));
+	GtkTextIter start,end;
+	gtk_text_buffer_get_start_iter(buffer, &start);
+	gtk_text_buffer_get_end_iter(buffer, &end);
+	return gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
 }
 
 void NodePropertiesDlg::RefreshKeywords()
@@ -199,7 +217,7 @@ void NodePropertiesDlg::KeywordAdd()
 
 		//check if valid keyword
 		if(std::string::npos != strWord.find(';')){
-			gtkMessageBox(_("Keyword must not contain \";\" character!"));
+			gtkMessageBox(_("Tag must not contain \";\" character!"));
 			return;
 		}
 
@@ -215,7 +233,7 @@ void NodePropertiesDlg::KeywordAdd()
 
 			if(value && 0 == strcmp(strWord.c_str(), value))
 			{
-				gtkMessageBox(_("This keyword was already added for this node!"));
+				gtkMessageBox(_("This tag was already added for this node!"));
 				g_free(value);
 				return;
 			}
@@ -277,7 +295,6 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 	GtkWidget *radio_ico_internal;
 	GtkWidget *radio_ico_custom;
 	GtkWidget *entry4;
-	GtkWidget *label23;
 	GtkWidget *comboboxentry3;
 	GtkWidget *hbox3;
 	GtkWidget *label24;
@@ -294,15 +311,17 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 	GtkWidget *label26;
 	GtkWidget *label_modified;
 	GtkWidget *chk_finished;
+	GtkWidget *scrolledwindow1;
 	
 	node_properties_dialog = gtk_dialog_new ();
 	gtk_window_set_title (GTK_WINDOW (node_properties_dialog), _("Node Properties"));
 	gtk_window_set_type_hint (GTK_WINDOW (node_properties_dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_dialog_set_has_separator (GTK_DIALOG(node_properties_dialog), FALSE);
 
 	dialog_vbox5 = GTK_DIALOG (node_properties_dialog)->vbox;
 	gtk_widget_show (dialog_vbox5);
 	
-	table4 = gtk_table_new (2, 2, FALSE);
+	table4 = gtk_table_new (2, 4, FALSE);
 	gtk_widget_show (table4);
 	gtk_box_pack_start (GTK_BOX (dialog_vbox5), table4, TRUE, TRUE, 0);
 
@@ -313,12 +332,24 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 		(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);
 	
-	entry_node_title = gtk_entry_new ();
-	gtk_widget_show (entry_node_title);
-	gtk_table_attach (GTK_TABLE (table4), entry_node_title, 1, 2, 0, 1,
-		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show (scrolledwindow1);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_table_attach (GTK_TABLE (table4), scrolledwindow1, 1, 4, 0, 1,
+		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (0), 0, 0);
 
+	entry_node_title = gtk_text_view_new(); //gtk_entry_new
+	gtk_widget_show (entry_node_title);
+#ifdef _NOKIA_MAEMO
+	gtk_widget_set_size_request(scrolledwindow1, -1, 25); // preserve the screen estate
+	gtk_widget_set_size_request(entry_node_title, -1, 25); // preserve the screen estate
+#else
+	gtk_widget_set_size_request(entry_node_title, -1, 30);
+#endif
+	gtk_container_add (GTK_CONTAINER (scrolledwindow1), entry_node_title);
+	g_editor = entry_node_title;
+
 	//attach accelerator
 	GtkAccelGroup *accel_group = gtk_accel_group_new ();
 	GClosure *closure = g_cclosure_new (G_CALLBACK (on_menu_insert_date_time), NULL, NULL);
@@ -342,74 +373,75 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 
 	label26 = gtk_label_new (_("Modified:"));
 	gtk_widget_show (label26);
-	gtk_table_attach (GTK_TABLE (table4), label26, 0, 1, 2, 3,
+	gtk_table_attach (GTK_TABLE (table4), label26, 2, 3, 1, 2,
 		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label26), 0, 0.5);
 
 	label_modified = gtk_label_new (_("unknown"));
 	gtk_widget_show (label_modified);
-	gtk_table_attach (GTK_TABLE (table4), label_modified, 1, 2, 2, 3,
+	gtk_table_attach (GTK_TABLE (table4), label_modified, 3, 4, 1, 2,
 		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label_modified), 0, 0.5);
 
-	label22 = gtk_label_new (_("Icon:"));
-	gtk_widget_show (label22);
-	gtk_table_attach (GTK_TABLE (table4), label22, 0, 1, 3, 4,
-		(GtkAttachOptions) (GTK_FILL),
-		(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (label22), 0, 0);
-
 	vbox10 = gtk_vbox_new (FALSE, 0);
 	gtk_widget_show (vbox10);
-	gtk_table_attach (GTK_TABLE (table4), vbox10, 1, 2, 3, 4,
+	gtk_table_attach (GTK_TABLE (table4), vbox10, 0, 4, 2, 3,
 		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (GTK_FILL), 0, 0);
 	
 	hbox2 = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox2);
 	gtk_box_pack_start (GTK_BOX (vbox10), hbox2, FALSE, FALSE, 0);
-	
-	table6 = gtk_table_new (4, 3, FALSE);
-	gtk_widget_show (table6);
-	gtk_box_pack_start (GTK_BOX (hbox2), table6, TRUE, TRUE, 0);
+
+	label22 = gtk_label_new (_("Icon:"));
+	gtk_widget_show (label22);
+	gtk_box_pack_start (GTK_BOX (hbox2), label22, TRUE, TRUE, 0);
+	gtk_misc_set_alignment (GTK_MISC (label22), 0, 0);
 
 	radio_ico_none = gtk_radio_button_new_with_mnemonic (NULL, _("None"));
 	gtk_widget_show (radio_ico_none);
-	gtk_table_attach (GTK_TABLE (table6), radio_ico_none, 0, 1, 0, 1,
-		(GtkAttachOptions) (GTK_FILL),
-		(GtkAttachOptions) (0), 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), radio_ico_none, TRUE, TRUE, 0);
 	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radio_ico_none), radio_ico_none_group);
 	radio_ico_none_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_ico_none));
 
 	radio_ico_internal = gtk_radio_button_new_with_mnemonic (NULL, _("Internal"));
 	gtk_widget_show (radio_ico_internal);
-	gtk_table_attach (GTK_TABLE (table6), radio_ico_internal, 0, 1, 1, 2,
-		(GtkAttachOptions) (GTK_FILL),
-		(GtkAttachOptions) (0), 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), radio_ico_internal, TRUE, TRUE, 0);
 	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radio_ico_internal), radio_ico_none_group);
 	radio_ico_none_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_ico_internal));
 
 	radio_ico_custom = gtk_radio_button_new_with_mnemonic (NULL, _("Custom"));
 	gtk_widget_show (radio_ico_custom);
-	gtk_table_attach (GTK_TABLE (table6), radio_ico_custom, 0, 1, 2, 3,
-		(GtkAttachOptions) (GTK_FILL),
-		(GtkAttachOptions) (0), 0, 0);
+	gtk_box_pack_start (GTK_BOX (hbox2), radio_ico_custom, TRUE, TRUE, 0);
 	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radio_ico_custom), radio_ico_none_group);
 	radio_ico_none_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio_ico_custom));
 
+	hbox3 = gtk_hbox_new (FALSE, 0);
+	gtk_widget_show (hbox3);
+	gtk_box_pack_start (GTK_BOX (vbox10), hbox3, FALSE, FALSE, 0);
+
+	image1 = gtk_image_new ();
+	gtk_widget_show (image1);
+	gtk_widget_set_size_request(image1, -1, 20);
+	gtk_box_pack_start (GTK_BOX (hbox3), image1, FALSE, FALSE, 20);
+
+	table6 = gtk_table_new (4, 3, FALSE);
+	gtk_widget_show (table6);
+	gtk_box_pack_start (GTK_BOX (hbox3), table6, TRUE, TRUE, 5);
+
 	comboboxentry2 = gtk_combo_box_new_text ();
 	gtk_widget_show (comboboxentry2);
 	gtk_table_attach (GTK_TABLE (table6), comboboxentry2, 1, 2, 1, 2,
-		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (GTK_FILL), 0, 0);
 	
 	entry4 = gtk_entry_new ();
 	gtk_entry_set_editable (GTK_ENTRY(entry4), FALSE);
 	gtk_widget_show (entry4);
 	gtk_table_attach (GTK_TABLE (table6), entry4, 1, 2, 2, 3,
-		(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (0), 0, 0);
 
 	button_xpm_select = gtk_button_new_with_mnemonic ("...");
@@ -418,20 +450,6 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (0), 0, 0);
 
-	label23 = gtk_label_new (_("Preview:"));
-	gtk_widget_show (label23);
-	gtk_table_attach (GTK_TABLE (table6), label23, 0, 1, 4, 5,
-		(GtkAttachOptions) (GTK_FILL),
-		(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);
-
-	image1 = gtk_image_new ();
-	gtk_widget_show (image1);
-	gtk_widget_set_size_request(image1, -1, 20);
-	gtk_table_attach (GTK_TABLE (table6), image1, 1, 2, 4, 5,
-		(GtkAttachOptions) (GTK_FILL),
-		(GtkAttachOptions) (GTK_FILL), 0, 0);
-	
 	label24 = gtk_label_new (_("Tags:"));
 	gtk_widget_show (label24);
 	gtk_table_attach (GTK_TABLE (table4), label24, 0, 1, 5, 6,
@@ -441,7 +459,7 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 
 	hbox3 = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox3);
-	gtk_table_attach (GTK_TABLE (table4), hbox3, 1, 2, 5, 6,
+	gtk_table_attach (GTK_TABLE (table4), hbox3, 1, 4, 5, 6,
 		(GtkAttachOptions) (GTK_FILL),
 		(GtkAttachOptions) (0), 0, 0);
 
@@ -471,6 +489,7 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 	gtk_widget_show (cancelbutton5);
 	gtk_dialog_add_action_widget (GTK_DIALOG (node_properties_dialog), cancelbutton5, GTK_RESPONSE_CANCEL);
 	GTK_WIDGET_SET_FLAGS (cancelbutton5, GTK_CAN_DEFAULT);
+	g_cancelbtn = cancelbutton5;
 	
 	okbutton5 = gtk_button_new_from_stock ("gtk-ok");
 	gtk_widget_show (okbutton5);
@@ -485,6 +504,7 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 	g_signal_connect (addbtn,		"clicked", G_CALLBACK (on_keyword_add_selected), this);
 	g_signal_connect (removebtn,		"clicked", G_CALLBACK (on_keyword_remove_selected), this);
 	g_signal_connect (chk_finished, "clicked",  G_CALLBACK (on_finished_checked), this);
+	g_signal_connect(entry_node_title,	"key_press_event",  G_CALLBACK (entry_keyboard_handler), this);
 
 	/* Store pointers to all widgets, for use by lookup_widget(). */
 	GLADE_HOOKUP_OBJECT_NO_REF (node_properties_dialog, node_properties_dialog, "node_properties_dialog");
@@ -500,7 +520,6 @@ GtkWidget* NodePropertiesDlg::create_node_properties_dialog ()
 	GLADE_HOOKUP_OBJECT (node_properties_dialog, radio_ico_internal, "radio_ico_internal");
 	GLADE_HOOKUP_OBJECT (node_properties_dialog, radio_ico_custom, "radio_ico_custom");
 	GLADE_HOOKUP_OBJECT (node_properties_dialog, entry4, "entry4");
-	GLADE_HOOKUP_OBJECT (node_properties_dialog, label23, "label23");
 	GLADE_HOOKUP_OBJECT (node_properties_dialog, hbox3, "hbox3");
 	GLADE_HOOKUP_OBJECT (node_properties_dialog, label24, "label24");
 	GLADE_HOOKUP_OBJECT (node_properties_dialog, comboboxentry3, "comboboxentry3");
@@ -534,6 +553,9 @@ void on_icon_none_clicked(GtkMenuItem *menuitem, gpointer user_data)
 		gtk_widget_set_sensitive(cbo2, FALSE);
 		gtk_widget_set_sensitive(entry4, FALSE);
 		gtk_widget_set_sensitive(button, FALSE);
+		gtk_widget_hide(cbo2);
+		gtk_widget_hide(entry4);
+		gtk_widget_hide(button);
 
 		pDlg->UpdatePreview();
 	}
@@ -554,6 +576,10 @@ void on_icon_internal_clicked(GtkMenuItem *menuitem, gpointer user_data)
 		gtk_widget_set_sensitive(entry4, FALSE);
 		gtk_widget_set_sensitive(button, FALSE);
 
+		gtk_widget_show(cbo2);
+		gtk_widget_hide(entry4);
+		gtk_widget_hide(button);
+
 		pDlg->UpdatePreview();
 	}
 }
@@ -573,6 +599,10 @@ void on_icon_custom_clicked(GtkMenuItem *menuitem, gpointer user_data)
 		gtk_widget_set_sensitive(entry4, TRUE);
 		gtk_widget_set_sensitive(button, TRUE);
 
+		gtk_widget_hide(cbo2);
+		gtk_widget_show(entry4);
+		gtk_widget_show(button);
+
 		pDlg->UpdatePreview();
 	}
 }
@@ -580,9 +610,28 @@ void on_icon_custom_clicked(GtkMenuItem *menuitem, gpointer user_data)
 void on_xpm_select_clicked(GtkMenuItem *menuitem, gpointer user_data)
 {
 	FileDialog dlg;
+	dlg.AddFilter(_("All supported icon formats (*.xpm,*.ico,*.png,*.jpg)"), "*.xpm|*.ico|*.png|*.jpg");
 	dlg.AddFilter(_("XPM icon file (*.xpm)"), "*.xpm");
+	dlg.AddFilter(_("Icon file (*.ico)"), "*.ico");
+	dlg.AddFilter(_("PNG file format (*.png)"), "*.png");
+	dlg.AddFilter(_("JPEG file format (*.jpg)"), "*.jpg");
+
+	//set initial directory from INI (store last used)
+	std::string strDefaultDir;
+	std::string strDir;
+	g_objIni.GetValue("Cache", "LastIconDir", strDir, "");
+	if(!strDir.empty() && 0 == access(strDir.c_str(), 00))
+		strDefaultDir = strDir;
+	else
+		strDefaultDir = GetHomeDir();
+
+	dlg.SetDirectory(strDefaultDir.c_str());
+
 	if(dlg.DoModal())
 	{
+		strDefaultDir = dlg.GetDirectory();
+		g_objIni.SetValue("Cache", "LastIconDir", strDefaultDir.c_str());
+
 		NodePropertiesDlg *pDlg = (NodePropertiesDlg *)user_data;
 		GtkWidget *entry4	= lookup_widget(pDlg->m_pDialog, "entry4");
 		gtk_entry_set_text(GTK_ENTRY(entry4), dlg.GetFilename());
@@ -615,7 +664,11 @@ void NodePropertiesDlg::UpdatePreview()
 	}
 	else if(ICON_CUSTOM == nType)
 	{
-		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size(GetIconValue(), 16, 16, NULL);
+		GdkPixbuf *pixbuf = NULL;
+		const char *szFile = GetIconValue();
+		if(szFile && strlen(szFile) > 0)
+			pixbuf = gdk_pixbuf_new_from_file_at_size(szFile, 16, 16, NULL);
+
 		if(pixbuf){
 			gtk_image_set_from_pixbuf (GTK_IMAGE(image1), pixbuf);
 			g_object_unref (G_OBJECT (pixbuf));
@@ -623,6 +676,9 @@ void NodePropertiesDlg::UpdatePreview()
 		else
 		{
 			gtk_image_set_from_file(GTK_IMAGE(image1), NULL);	//clear image
+
+			if(szFile && strlen(szFile) > 0)
+				gtkMessageBox(_("Failed to load node icon file!"));
 		}
 	}
 	else
@@ -691,9 +747,47 @@ const char *combo_get_text(GtkWidget *cbo3)
 			return value;
 			//g_free(value);
 		}
+		else
+			return "";	
 	}
 	else //TOFIX get text from entry
 		return "";
  #endif
 #endif
 }
+
+
+gint entry_keyboard_handler(GtkWidget *widget, GdkEventKey *event)
+{
+	if(event->keyval == GDK_Return)
+	{
+		if(event->state & GDK_CONTROL_MASK)
+		{
+			return FALSE;
+		}
+		else
+		{
+			if(g_bCloseDlgOnTextEnter){
+				gtk_dialog_response(GTK_DIALOG(g_pNodePropWidget), GTK_RESPONSE_OK);
+				return TRUE;
+			}
+		}
+	}
+    else if(event->keyval == GDK_Tab)
+	{
+		if(event->state & GDK_CONTROL_MASK)
+		{
+			//insert tab character
+			GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(g_editor));
+			gtk_text_buffer_insert_at_cursor(buf, "\t", -1);
+			return TRUE;
+		}
+		else
+		{
+            gtk_widget_grab_focus (g_cancelbtn); // pass focus forward
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+} 
