@@ -7,6 +7,7 @@
 ////////////////////////////////////////////////////////////////////////////
 
 #include "TextView.h"
+#include "config.h"
 
 //declare signal handlers
 void on_textview_edited (GtkTextBuffer *, gpointer data);
@@ -17,6 +18,13 @@ gboolean textview_mouse_click(GtkWidget *widget, GdkEventButton *event, gpointer
 gboolean textview_mouse_moved(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
 void textview_cursor_moved(GtkTextView *widget, GtkMovementStep *step, gint count, gboolean extend_selection, gpointer user_data);
 void textview_selection_changed(GtkTextBuffer *textbuffer, GtkTextIter *arg1, GtkTextMark *arg2, gpointer user_data);
+gboolean textview_on_drop(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, guint time, gpointer user_data);
+void drag_data_received_handl(GtkWidget *widget, GdkDragContext *context, gint x, gint y, GtkSelectionData *selection_data, guint target_type, guint time, gpointer data);
+
+extern GdkColor g_linkColor;
+
+GdkAtom atomTextUriList;
+GdkAtom atomString;
 
 TextView::TextView()
 {
@@ -38,7 +46,21 @@ void TextView::Create()
 	g_signal_connect (G_OBJECT (textview1), "key_press_event", G_CALLBACK (textview_keyboard_handler), NULL);
 	g_signal_connect (G_OBJECT (textview1), "button-press-event", G_CALLBACK (textview_mouse_click), NULL);
 	g_signal_connect (G_OBJECT (textview1), "motion-notify-event", G_CALLBACK (textview_mouse_moved), this);
+	g_signal_connect (G_OBJECT (textview1), "drag-drop",			G_CALLBACK (textview_on_drop), this);
+	g_signal_connect (G_OBJECT (textview1), "drag-data-received",	G_CALLBACK (drag_data_received_handl), this);
 
+	//make the "textview1" a DnD destination
+	gtk_drag_dest_add_text_targets(GTK_WIDGET(textview1));
+	gtk_drag_dest_add_uri_targets(GTK_WIDGET(textview1));
+
+	atomTextUriList = gdk_atom_intern("text/uri-list", FALSE);
+	atomString		= gdk_atom_intern("STRING", FALSE);
+
+#ifdef _NOKIA_MAEMO
+	gtk_widget_tap_and_hold_setup(GTK_WIDGET(textview1), NULL, NULL, (GtkWidgetTapAndHoldFlags)(GTK_TAP_AND_HOLD_PASS_PRESS /*GTK_TAP_AND_HOLD_NONE | GTK_TAP_AND_HOLD_NO_INTERNALS*/));
+	//g_signal_connect(treeview1, "tap-and-hold", G_CALLBACK(on_popup_menu), NULL);
+#endif
+ 
 	g_signal_connect(buffer, "changed", G_CALLBACK(on_textview_edited),  NULL); //track buffer changes
 
 	//events to handle undo in textview
@@ -287,6 +309,11 @@ void TextView::RestoreSelectionBounds(unsigned int cursor, unsigned int selectio
 void TextView::SetEditable(bool bEditable)
 {
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(m_pWidget), bEditable);
+
+	if(bEditable)
+		GTK_WIDGET_SET_FLAGS(m_pWidget, GTK_CAN_FOCUS);
+	else
+		GTK_WIDGET_UNSET_FLAGS(m_pWidget, GTK_CAN_FOCUS);
 }
 
 bool TextView::IsEditable()
@@ -333,9 +360,12 @@ void TextView::SetTextUnderlined(int nStart, int nEnd)
 	gtk_text_buffer_get_iter_at_offset(buffer1, &cursIter, nStart);
 	gtk_text_buffer_get_iter_at_offset(buffer1, &boundIter, nEnd);
 
+	char szColor[30];
+	sprintf (szColor, "#%04x%04x%04x", g_linkColor.red, g_linkColor.green, g_linkColor.blue);
+
 	//"linktag"
 	GtkTextTag *tag = gtk_text_buffer_create_tag (buffer1, NULL,
-			"foreground", "blue",
+			"foreground", szColor,
 			"underline", PANGO_UNDERLINE_SINGLE,
 			"underline-set", TRUE, 
 			NULL);
