@@ -23,18 +23,30 @@ public:
 	void Create();
 	
 	void Clear();
+	void SetEditable(bool bEditable);
+	bool IsEditable();
+
 	void DeleteText(int nOffset, int nLength);
 
-	gchar *GetText();	//requires user to free result with g_free
+	gchar *GetText();			//requires user to free result with g_free
+	gchar *GetSelectedText();	//requires user to free result with g_free
+
 	void SetText(const gchar *szText);
 	void InsertText(int nOffset, const gchar *szText);
+	void ReplaceText(int nStart, int nEnd, const char *szNewTxt);
 
 	int  GetSelectionEnd();
-	void GetSelectionBounds(unsigned int& cursor, unsigned int& selection);
+	void GetSelectionBounds(int& cursor, int& selection);
 	void RestoreSelectionBounds(unsigned int cursor, unsigned int selection);
 	void SelectRange(int nStart, int nEnd);
 	void EnsureVisible(int nOffset);
 
+	void SetTextUnderlined(int nStart, int nEnd);
+	void RemoveTextStyles(int nStart, int nEnd);
+
+	void SetTabWidth(int nWidth);	//width in pixels
+	int	 CalcTabWidthInPixels(int nNumChars);
+
 	bool IsWrapped();
 	bool GetModified();
 	void SetModified(bool bModified);
@@ -42,7 +54,7 @@ public:
 	void ClipboardCut();
 	void ClipboardCopy();
 	void ClipboardPaste();
-	void ClipboardDelete();
+	void DeleteSelection();
 
 	void SetFocus();
 
