@@ -13,6 +13,7 @@
 #include "lib/LinkInfo.h"
 #include "lib/NoteDocument.h"
 
+void edit_node_title();
 void UpdateTextFromScreen();
 void UpdateUndoRedoMenus();
 void ExecuteLink(LinkInfo &link);
@@ -28,8 +29,11 @@ void cell_edited_cancel_callback (GtkCellRenderer *renderer, gpointer user_data)
 
 void on_tree_row_click (GtkTreeView *treeview, gpointer user_data);
 void on_tree_row_collapsed (GtkTreeView *tree_view, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
+void on_tree_row_expanded (GtkTreeView *tree_view, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
 gint treeview_keyboard_handler(GtkWidget *widget, GdkEventKey *event);
 gboolean on_tree_button_release(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+gint on_popup_menu(GtkWidget *widget, gpointer data);
+void treeview_show_popup_menu();
 
 void on_dnd_row_deleted  (GtkTreeModel *treemodel, GtkTreePath *arg1, gpointer user_data);
 void on_dnd_row_inserted (GtkTreeModel *treemodel, GtkTreePath *arg1, GtkTreeIter *arg2, gpointer user_data);
@@ -75,7 +79,14 @@ void TreeView::Create()
 	g_signal_connect (treeview1, "button-release-event", G_CALLBACK (on_tree_button_release), this);
 	g_signal_connect (treeview1, "button-press-event",   G_CALLBACK (on_tree_button_release), this);
 	g_signal_connect (treeview1, "row-collapsed",	     G_CALLBACK (on_tree_row_collapsed), this);
-	
+	g_signal_connect (treeview1, "row-expanded",	     G_CALLBACK (on_tree_row_expanded), this);
+	g_signal_connect (treeview1, "popup-menu",			 G_CALLBACK (on_popup_menu), this);
+
+#ifdef _NOKIA_MAEMO
+	gtk_widget_tap_and_hold_setup(GTK_WIDGET(treeview1), NULL, NULL, (GtkWidgetTapAndHoldFlags)(GTK_TAP_AND_HOLD_NONE | GTK_TAP_AND_HOLD_NO_INTERNALS));
+	g_signal_connect(treeview1, "tap-and-hold", G_CALLBACK(on_popup_menu), NULL);
+#endif
+
 	//pack two renderers in the single column
 	column = gtk_tree_view_column_new();
 
@@ -586,88 +597,6 @@ void on_dnd_end (GtkWidget *view, GdkDragContext *context, gpointer data)
 
 gboolean on_tree_button_release(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
-#ifdef _ZAURUS_BUILD
-	TreeView *pTree = (TreeView *)user_data;
-	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
-
-	static GtkTreeIter iterSelected;
-	static bool bIterSelectedValid = false;
-
-	if(event->button == 1 && event->type == GDK_BUTTON_PRESS)
-	{
-		//skip if already in edit mode
-		if(pTree && pTree->m_bCellEdited){
-			bIterSelectedValid = false;	//reset info
-			return FALSE;
-		}
-
-		//get currently selected item
-		GtkTreeSelection* treesel = gtk_tree_view_get_selection (GTK_TREE_VIEW(widget));
-		GtkTreeIter  child;
-		gtk_tree_selection_get_selected(treesel, &model, &child);
-		iterSelected = child;
-		bIterSelectedValid = true;
-	}
-	else if(event->button == 1 && event->type == GDK_BUTTON_RELEASE)
-	{
-		//get currently selected item
-		GtkTreeSelection* treesel = gtk_tree_view_get_selection (GTK_TREE_VIEW(widget));
-		GtkTreeIter  child;
-		if(!gtk_tree_selection_get_selected(treesel, &model, &child))
-			return FALSE;
-
-		//get tree item that is below mouse pointer
-		int cell_x, cell_y;
-		GtkTreeViewColumn *column = NULL;
-		GtkTreePath *path = NULL;
-		if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget),
-										 (int)event->x, (int)event->y, &path, &column, &cell_x, &cell_y))
-		{
-			GtkTreePath *path1, *path2;
-			path1 = gtk_tree_model_get_path(model, &child);
-			path2 = (bIterSelectedValid) ? gtk_tree_model_get_path(model, &iterSelected) : NULL;
-
-			if( path && path1 && path2 &&
-				0 == gtk_tree_path_compare(path, path1) &&
-				0 == gtk_tree_path_compare(path, path2))
-			{
-				//equal paths -> we generated mouse button released event above previously selected item
-
-				//now test if clicked inside of cell area
-				// only text click should count - not expander click or empty area click
-				GdkRectangle rect;
-				gtk_tree_view_get_cell_area(GTK_TREE_VIEW(widget), path, column, &rect);
-
-				if( cell_x > rect.x &&	
-					cell_x < rect.width && 	
-					cell_y > 0 &&
-					cell_y < rect.height)
-				{
-					int nIdx = GetSelectedNodeIdx();
-					if(nIdx >= 0){ //execute link if exists
-						NoteNode &node = g_doc.GetNodeByIdx(nIdx);
-						if(node.m_objLink.IsValid())
-						{
-							ExecuteLink(node.m_objLink);
-
-							gtk_tree_path_free(path);
-							gtk_tree_path_free(path1);
-							gtk_tree_path_free(path2);
-							return FALSE;
-						}
-					}
-
-					pTree->EditLabel();	//trigger label edit mode
-				}
-			}
-			
-			gtk_tree_path_free(path);
-			gtk_tree_path_free(path1);
-			gtk_tree_path_free(path2);
-		}
-	}
-	else 
-#endif
 	if(event->button == 1 && event->type == GDK_2BUTTON_PRESS)
 	{
 		int nIdx = GetSelectedNodeIdx();
@@ -679,3 +608,24 @@ gboolean on_tree_button_release(GtkWidget *widget, GdkEventButton *event, gpoint
 
 	return FALSE;
 }
+
+//this handler enables keyboard-only access to the context menu
+// -> use shortcut: Shift + F10
+gint on_popup_menu(GtkWidget *widget, gpointer data)
+{
+	treeview_show_popup_menu();
+	return TRUE;
+}
+
+void TreeView::WrapTreeToSize(int nSize)
+{
+	GtkTreeView *treeview = (GtkTreeView *)m_pWidget;
+	GtkTreeViewColumn *column1 = gtk_tree_view_get_column(treeview, 0);
+	GList *list1 = gtk_tree_view_column_get_cell_renderers(column1);
+	GtkCellRendererText *renderer = (GtkCellRendererText *)g_list_nth_data(list1, SCREEN_IDX_TEXT);
+
+	g_object_set (G_OBJECT (renderer), 
+		"wrap-mode", PANGO_WRAP_CHAR,
+		"wrap-width", nSize, 
+		NULL);
+}
