@@ -10,9 +10,6 @@
 #ifndef __SINGLEINSTANCE_H__
 #define __SINGLEINSTANCE_H__
 
-#include <sys/ipc.h>
-#include <sys/shm.h>
-
 class CSingleInstance
 {
 public:
