@@ -6,6 +6,15 @@
 // File: Class to register document format (*.ext) to be handled by given application 
 ////////////////////////////////////////////////////////////////////////////
 
+#if _MSC_VER > 1000
+  #pragma warning(disable: 4996)	//ignore deprecated functions warning
+#endif
+
+#ifdef _WIN32
+ #include <io.h>	//_access
+ #define access _access
+#endif
+
 #include "RegisterExtension.h"
 #include <windows.h>
 
@@ -112,14 +121,39 @@ bool RegisterFileExtension::IsRegistered(const char *szExt)
 	if(ERROR_SUCCESS == RegOpenKey(HKEY_CLASSES_ROOT, szExt, &hKey))
 	{
 		//check if our program is the main handler
-		char szValue[256];
+		char szValue[2048];
 		DWORD dwValueSize = sizeof(szValue);
-        if(ERROR_SUCCESS == RegQueryValueEx(hKey, NULL, NULL, NULL, (LPBYTE)szValue, &dwValueSize))
+		LONG lRes = RegQueryValueEx(hKey, NULL, NULL, NULL, (LPBYTE)szValue, &dwValueSize);
+        if(ERROR_SUCCESS == lRes)
 		{
 			if(0 == strcmp(m_strName.c_str(), szValue))
 			{
+				//check if the the app path for this format is valid one
+				std::string strKey2 = m_strName;
+				strKey2 += "\\Shell\\open\\command";
+
+				HKEY hKey2;
+				lRes = RegOpenKey(HKEY_CLASSES_ROOT, strKey2.c_str(), &hKey2);
+				if(ERROR_SUCCESS == lRes)
+				{
+					dwValueSize = sizeof(szValue);
+
+					lRes = RegQueryValueEx(hKey2, NULL, NULL, NULL, (LPBYTE)szValue, &dwValueSize);
+					if(ERROR_SUCCESS == lRes)
+					{
+						std::string strApp(szValue);
+						strApp = strApp.substr(0, strApp.size()-3); // strip " %1" frpm cmd string
+
+						//check if the the value - app path is valid one
+						if(0 != access(strApp.c_str(), 00))
+							lRes = 1000;	// some error
+					}
+
+					RegCloseKey(hKey2);
+				}
+
 				RegCloseKey(hKey);
-				return true;
+				return (ERROR_SUCCESS == lRes);
 			}
 		}
 
