@@ -18,6 +18,9 @@ void on_open1_activate       (GtkMenuItem *menuitem, gpointer user_data);
 void on_save1_activate       (GtkMenuItem *menuitem, gpointer user_data);
 void on_save_as1_activate    (GtkMenuItem *menuitem, gpointer user_data);
 void on_reload1_activate     (GtkMenuItem *menuitem, gpointer user_data);
+void on_lock_document_activate(GtkMenuItem *menuitem, gpointer user_data);
+void on_change_pass1_activate(GtkMenuItem *menuitem, gpointer user_data);
+void on_read_only_activate	 (GtkMenuItem *menuitem, gpointer user_data);
 void on_quit1_activate       (GtkMenuItem *menuitem, gpointer user_data);
 void on_cut1_activate        (GtkMenuItem *menuitem, gpointer user_data);
 void on_copy1_activate       (GtkMenuItem *menuitem, gpointer user_data);
@@ -60,6 +63,7 @@ void on_text_italic_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_text_underline_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_text_strikethrough_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_text_color_activate (GtkMenuItem *menuitem, gpointer user_data);
+void on_text_background_color_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_remove_format_activate (GtkMenuItem *menuitem, gpointer user_data);
 void on_view_text_only_mode (GtkMenuItem *menuitem, gpointer user_data);
 void on_view_calc_word_count (GtkMenuItem *menuitem, gpointer user_data);
@@ -90,7 +94,7 @@ void on_textview_populate_popup(GtkWidget *widget, GtkMenu *menu, void *user_dat
 
 void UpdateUndoRedoMenus();
 void SetNodeTitle(int nIdx, const char *szTitle);
-void InsertNodeText(int nIdx, int nOffset, const char *szText);
+void InsertNodeText(int nIdx, int nOffset, const char *szText, std::vector<PixInfo> *pLstPictures = NULL);
 void DeleteNodeText(int nIdx, int nOffset, int nLength);
 
 void on_delete_text (GtkTextBuffer *textbuffer,
@@ -103,7 +107,7 @@ void on_insert_text (GtkTextBuffer *textbuffer,
                      gint arg3,
                      gpointer user_data);
 
-void RefreshTextFormat(NoteNode &node);
+void RefreshTextFormat(NoteNode &node, bool bAddPictures = true);
 void UpdateTextFromScreen();
 void RefreshMainTitle();
 void TreeIterFromID(GtkTreeIter  &iter, int nNodeID);
@@ -121,9 +125,19 @@ gboolean on_window_delete_event(GtkWidget *widget, GdkEvent *event, gpointer dat
 void restart_autosave();
 bool autosave_check_crash();
 void autosave_shutdown();
+void remove_autosave_unencrypted();
 
 void restart_enc_unload_timer();
 
 void do_sort_branch(int nParentIdx, bool bAscending);
 
+void set_read_only(bool bSet);
+bool get_read_only();
+
+void on_full_screen_activate (GtkMenuItem *menuitem, gpointer user_data);
+void edit_node_title();
+gboolean date_time_paste_to_text_widget(GtkWidget *widget);
+
+void on_node_toggle_finished(GtkMenuItem *menuitem, gpointer user_data);
+
 #endif
