@@ -10,14 +10,17 @@
 #define CONFIG_H__
 
 #define APP_NAME_STR "NoteCase" 
-#define APP_VER_STR "1.5.6"
+#define APP_VER_STR "1.7.9"
 
 #define PACKAGE "notecase"	//NLS package definition
 #define UNDO_LIST_SIZE 10
-
-//#define _ZAURUS_BUILD		//Zaurus specific code
+#define GENERATE_CRASHLOG	// comment this to suppress crash log generation
 
 #ifndef _WIN32
+//#define _ZAURUS_BUILD
+//#define _OSX_BUILD
+//#define _NOKIA_MAEMO
+
 #define INSTALL_PREFIX "/usr"
 #define HAVE_GNOME_VFS
 #endif
