@@ -7,11 +7,20 @@
 ////////////////////////////////////////////////////////////////////////////
 
 #include "FileDialog.h"
+#include "../config.h"
 #include <string>
 #include <gdk/gdkkeysyms.h>
 #include "../support.h"
 #include "../lib/debug.h"
 
+#ifdef _NOKIA_MAEMO
+ #if _NOKIA_MAEMO == 1
+  #include <hildon-widgets/hildon-file-chooser-dialog.h>
+ #else
+  #include <hildon/hildon-file-chooser-dialog.h>
+ #endif
+#endif
+
 #if !GTK_CHECK_VERSION(2,4,0) //minimal version for new type of dialog
    #define _OLD_DIALOG
 #else
@@ -92,6 +101,29 @@ FileDialog::FileDialog(bool bOpen, bool bOnlyFolders, bool bExpand, const char *
 	gtk_container_add (GTK_CONTAINER(dialog_action_area6), okbutton6);
 	GTK_WIDGET_SET_FLAGS (okbutton6, GTK_CAN_DEFAULT);
  #else
+  #ifdef _NOKIA_MAEMO
+	GtkFileChooserAction action;
+	if(bOpen)
+		action = (bOnlyFolders)? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER : GTK_FILE_CHOOSER_ACTION_OPEN;
+	else
+		action = (bOnlyFolders)? GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER : GTK_FILE_CHOOSER_ACTION_SAVE;
+	
+	m_pDialog = hildon_file_chooser_dialog_new(GTK_WINDOW (pParent), action);
+	gtk_widget_show_all(GTK_WIDGET(m_pDialog));
+
+	gtk_window_set_modal (GTK_WINDOW (m_pDialog), TRUE);
+#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (m_pDialog), TRUE);
+#endif
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (m_pDialog), TRUE);
+	gtk_window_set_type_hint (GTK_WINDOW (m_pDialog), GDK_WINDOW_TYPE_HINT_DIALOG);
+#if GTK_CHECK_VERSION(2,4,0) //new API TOFIX set proper version
+	#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
+	  //gtk_window_set_keep_above(GTK_WINDOW (m_pDialog), TRUE);
+	#endif
+#endif
+
+  #else
 	if(bOpen)
 		m_pDialog = gtk_file_chooser_dialog_new(szDlgTitle, GTK_WINDOW (pParent), 
 			(bOnlyFolders)? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER : GTK_FILE_CHOOSER_ACTION_OPEN, 
@@ -124,8 +156,21 @@ FileDialog::FileDialog(bool bOpen, bool bOnlyFolders, bool bExpand, const char *
 			g_list_free(children1);
 			g_list_free(children);
 		}
-
 	}
+
+	gtk_window_set_modal (GTK_WINDOW (m_pDialog), TRUE);
+#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (m_pDialog), TRUE);
+#endif
+	gtk_window_set_skip_pager_hint (GTK_WINDOW (m_pDialog), TRUE);
+	gtk_window_set_type_hint (GTK_WINDOW (m_pDialog), GDK_WINDOW_TYPE_HINT_DIALOG);
+#if GTK_CHECK_VERSION(2,4,0) //new API TOFIX set proper version
+	#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
+	  //gtk_window_set_keep_above(GTK_WINDOW (m_pDialog), TRUE);
+	#endif
+#endif
+	
+  #endif
  #endif
 #endif
 }
