@@ -16,11 +16,16 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <signal.h>
+#include <errno.h>
+
 #ifndef _WIN32
  #include <unistd.h>
 #else
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
 #endif 
 
@@ -52,6 +57,11 @@
 #ifdef _WIN32
  #include "_win/RegisterExtension.h"
  #include <windows.h>
+#else
+ #include <sys/types.h>	//getpid
+ #include <unistd.h>
+ #include <signal.h>	//kill
+ #include <errno.h>
 #endif
 
 extern NoteDocument g_doc;
@@ -70,6 +80,7 @@ void register_extensions_check();
 
 void cell_edited_callback(GtkCellRendererText *cell, gchar *path_string, gchar *new_text, gpointer user_data);
 void on_MRU_activate (GtkMenuItem *menuitem, gpointer user_data);
+void treeview_show_popup_menu();
 
 //
 // MRU list
@@ -82,132 +93,7 @@ gboolean treeview1_popup_menu_event_handler (GtkWidget *widget, GdkEventButton *
 	// Ignore double-clicks and triple-clicks
 	if (event->button == 3 && event->type == GDK_BUTTON_RELEASE)
 	{
-		GtkWidget *menu, *submenu;
-		GtkWidget *menu_item, *move_item, *sort_item;
-		int button, event_time;
-	
-		menu = gtk_menu_new ();
-		//g_signal_connect (menu, "deactivate", G_CALLBACK(gtk_widget_destroy), NULL);
-	
-		GtkAccelGroup *accel_group = gtk_accel_group_new();
-		gtk_menu_set_accel_group (GTK_MENU (menu), accel_group);
-
-		// ... add menu items with accelerators ... 
-		menu_item = gtk_menu_item_new_with_label(_("Insert Node"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_insert_node), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Insert, (GdkModifierType)0, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-		menu_item = gtk_menu_item_new_with_label(_("Insert Child Node"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_insert_child_node), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Insert, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-		menu_item = gtk_menu_item_new_with_label(_("Delete Node"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_delete_node), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Delete, (GdkModifierType)0, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-	
-		menu_item = gtk_menu_item_new_with_label(_("Rename Node"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_rename_node), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_F2, (GdkModifierType)0, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, 'M', (GdkModifierType)GDK_CONTROL_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-		move_item = gtk_menu_item_new_with_label(_("Move Node"));
-		gtk_menu_append(menu, move_item);
-		gtk_widget_show (move_item);  // Show the widget 
-
-		sort_item = gtk_menu_item_new_with_label(_("Sort"));
-		gtk_menu_append(menu, sort_item);
-		gtk_widget_show (sort_item);  // Show the widget 
-
-		menu_item = gtk_menu_item_new_with_label(_("Expand All"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_expand_all), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-
-		menu_item = gtk_menu_item_new_with_label(_("Collapse All"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_collapse_all), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-
-		menu_item = gtk_menu_item_new_with_label(_("Node Properties"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_node_properties), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Return, (GdkModifierType)GDK_CONTROL_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-		menu_item = gtk_menu_item_new_with_label(_("Link wizard"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_link_wizard_activate), NULL);
-		gtk_menu_append(menu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-
-		//"move node" submenu
-		submenu = gtk_menu_new ();
-	
-		menu_item = gtk_menu_item_new_with_label(_("Up"));
-		gtk_menu_append(submenu, menu_item);
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_up), NULL);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Up, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-	
-		menu_item = gtk_menu_item_new_with_label(_("Down"));
-		gtk_menu_append(submenu, menu_item);
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_down), NULL);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Down, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-	
-		menu_item = gtk_menu_item_new_with_label(_("Left"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_left), NULL);
-		gtk_menu_append(submenu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Left, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-	
-		menu_item = gtk_menu_item_new_with_label(_("Right"));
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_move_right), NULL);
-		gtk_menu_append(submenu, menu_item);
-		gtk_widget_show (menu_item);  // Show the widget 
-		gtk_widget_add_accelerator (menu_item, "activate", accel_group, GDK_Right, (GdkModifierType)GDK_SHIFT_MASK, (GtkAccelFlags)(GTK_ACCEL_VISIBLE));
-
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (move_item), submenu);
-	
-		//"sort node" submenu
-		submenu = gtk_menu_new ();
-	
-		menu_item = gtk_menu_item_new_with_label(_("Sort Children Ascending"));
-		gtk_menu_append(submenu, menu_item);
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_child_ascending), NULL);
-		gtk_widget_show (menu_item);  // Show the widget 
-
-		menu_item = gtk_menu_item_new_with_label(_("Sort Children Descending"));
-		gtk_menu_append(submenu, menu_item);
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_child_descending), NULL);
-		gtk_widget_show (menu_item);  // Show the widget 
-
-		menu_item = gtk_menu_item_new_with_label(_("Sort Root Ascending"));
-		gtk_menu_append(submenu, menu_item);
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_root_ascending), NULL);
-		gtk_widget_show (menu_item);  // Show the widget 
-
-		menu_item = gtk_menu_item_new_with_label(_("Sort Root Descending"));
-		gtk_menu_append(submenu, menu_item);
-		g_signal_connect(menu_item, "activate",	G_CALLBACK (on_menu_sort_root_descending), NULL);
-		gtk_widget_show (menu_item);  // Show the widget 
-
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (sort_item), submenu);
-
-		if (event)
-			event_time = event->time;
-		else
-			event_time = gtk_get_current_event_time ();
-		button = 0;	//FIX: allow mouse button to trigger the submenu
-	
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, event_time);
+		treeview_show_popup_menu();
 	}
 	return FALSE;
 }
@@ -317,8 +203,11 @@ void cell_edited_callback(GtkCellRendererText *cell, gchar *path_string, gchar *
 
 void on_textview_edited( GtkTextBuffer *widget, gpointer data)
 {
-	g_doc.SetModified(true);
-	RefreshMainTitle();
+	bool bModified = g_doc.IsModified();
+	if(!bModified){
+		g_doc.SetModified(true);
+		RefreshMainTitle();
+	}
 }
 
 //
@@ -393,7 +282,16 @@ void on_MRU_activate (GtkMenuItem *menuitem, gpointer user_data)
 	{
 		std::string strFile = g_objMRU.GetFile(nIdx);
 		
-		int nResult = load_file(strFile.c_str());
+		//allow password being entered up to 3 times
+		int nResult = 0;
+		int nTryCnt = 0;
+		while(nTryCnt < 3){
+			nResult = load_file(strFile.c_str());
+			nTryCnt ++;
+			if(DOC_LOAD_WRONG_PASSWORD != nResult)
+				break;
+		}
+
 
 		//do not delete MRU for file that exists, but failed to open
 		g_objMRU.Change(strFile.c_str(), (DOC_LOAD_NOT_FOUND != nResult));
@@ -511,7 +409,7 @@ int gtkMessageBox(const char *szText, int nButtons, int nIcon)
 
 #if GTK_CHECK_VERSION(2,4,0) //new API TOFIX set proper version
 	#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
-		gtk_window_set_keep_above(GTK_WINDOW (msgbox), TRUE);
+		//gtk_window_set_keep_above(GTK_WINDOW (msgbox), TRUE);
 	#endif
 #endif
 	gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
@@ -568,7 +466,7 @@ int gtkMessageBoxYNC(const char *szText)
 #endif
 #if GTK_CHECK_VERSION(2,4,0) //new API TOFIX set proper version
 	#ifndef _WIN32  //TOFIX API is buggy on Win32 (kills modal dialog state)
-		gtk_window_set_keep_above(GTK_WINDOW (msgbox), TRUE);
+		//gtk_window_set_keep_above(GTK_WINDOW (msgbox), TRUE);
 	#endif
 #endif
 	gtk_window_set_resizable (GTK_WINDOW (msgbox), FALSE);
@@ -658,6 +556,8 @@ const char *GetLoadErrorString(int nErrCode)
 			return _("Error when parsing document (bad formatting)!");
 		case DOC_LOAD_ERROR:
 			return _("Failed to load the file!");
+		case DOC_LOAD_PERMISSIONS:
+			return _("Inadequate file permissions!");
 		case DOC_LOAD_NOT_SUPPORTED:
 			return _("Load operation is not supported for this format!");
 	}
@@ -721,3 +621,28 @@ void HideBusyCursor()
 		g_bBusyCursor = false;
 	}
 }
+
+unsigned int GetCurrentPID()
+{
+#ifdef _WIN32
+	return GetCurrentProcessId();
+#else
+	return getpid();
+#endif
+}
+
+bool IsPIDRunning(unsigned int nPID)
+{
+#ifdef _WIN32
+	HANDLE hProcess = OpenProcess(PROCESS_ALL_ACCESS, FALSE, nPID);
+	if(NULL == hProcess)
+		return false;
+	CloseHandle(hProcess);
+#else
+	if(0 != kill(nPID, 0))
+		if(errno == ESRCH)
+			return false;
+#endif
+
+	return true;
+}
