@@ -26,10 +26,14 @@ const char *GetSaveErrorString(int nErrCode);
 void ShowBusyCursor();
 void HideBusyCursor();
 
+unsigned int GetCurrentPID();
+bool IsPIDRunning(unsigned int nPID);
+
 
 // main.cpp
 const char *calculate_autosave_filename();
 const char *calculate_autosave_filename1();
+const char *get_autosave_dir();
 std::string GetHelpFile();
 const char *GetIniFile();
 
