@@ -10,6 +10,8 @@
 #ifndef DOCACTIONBASE_H_
 #define DOCACTIONBASE_H_
 
+#include "NoteDocument.h"
+
 #if _MSC_VER > 1000
 #pragma once
 #endif // _MSC_VER > 1000
@@ -17,11 +19,16 @@
 class DocActionBase 
 {
 public:
-	DocActionBase(){};
+	DocActionBase(){ m_pDoc = NULL; };
 	virtual ~DocActionBase(){};
 
-	virtual void Redo() = 0;
+	virtual void Exec(bool bInteractive = false) = 0;
 	virtual void Undo() = 0;
+
+	void SetDoc(NoteDocument &doc){ m_pDoc = &doc; }
+
+protected:
+	NoteDocument *m_pDoc;	//pointer to the document
 };
 
 #endif // DOCACTIONBASE_H_
