@@ -74,14 +74,18 @@ int EnumDirectory(const char *szPath, FN_ENUMDIR func, void *userdata, int nFilt
 			if(0 == (fileinfo.attrib & _A_SUBDIR))	// if file
 			{
 				if(nFilter & ENUM_LST_FILES)
-					if(!func(szFullPath, userdata))
+					if(!func(szFullPath, userdata)){
+						_findclose(lHandle);
 						return ENUM_ERR_OK;	//abort request
+					}
 			}
 			else	// if directory
 			{
 				if(nFilter & ENUM_LST_DIRS)
-					if(!func(szFullPath, userdata))
+					if(!func(szFullPath, userdata)){
+						_findclose(lHandle);
 						return ENUM_ERR_OK;	//abort request
+					}
 						
 				if(nFilter & ENUM_RECURSIVE)
 				{
@@ -98,7 +102,8 @@ int EnumDirectory(const char *szPath, FN_ENUMDIR func, void *userdata, int nFilt
         nProceed = _findnext(lHandle, &fileinfo); // find next
     }
 
-	_findclose(lHandle);
+	if(-1L != lHandle)
+		_findclose(lHandle);
 	return ENUM_ERR_OK;
 }
 
