@@ -9,22 +9,30 @@
 #include "File64.h"
 #include "debug.h"
 
+#include <stdlib.h>
+
 #ifdef _WIN32
+ #ifndef __MINGW32__
+  #include <io.h>
+  #include <glib.h>
+  #include <glib/gstdio.h>
+ #else 
+  #include <sys/types.h>
+  #include <sys/stat.h>
+  #include <unistd.h>
+ #endif
 #else
  #include <sys/types.h>
  #include <sys/stat.h>
- #include <fcntl.h>
  #include <unistd.h>
- #include <errno.h>
 #endif
 
+#include <fcntl.h>
+#include <errno.h>
+
 File64::File64()
 {
-#ifdef _WIN32
-	m_hFile = INVALID_HANDLE_VALUE;
-#else
 	m_nFile	= -1;
-#endif
 }
 
 File64::~File64()
@@ -34,46 +42,6 @@ File64::~File64()
 
 bool File64::Open(const char *szPath, unsigned long nFlags)
 {
-#ifdef _WIN32
-	DWORD dwDesiredAccess = 0;
-	if(nFlags & F64_READ)
-		dwDesiredAccess |= GENERIC_READ;
-	if(nFlags & F64_WRITE)
-		dwDesiredAccess |= GENERIC_WRITE;
-
-	DWORD dwShareMode = 0;
-	if(nFlags & F64_SHARE_READ)
-		dwShareMode |= FILE_SHARE_READ;
-	if(nFlags & F64_SHARE_WRITE)
-		dwShareMode |= FILE_SHARE_WRITE;
-
-	DWORD dwCreationDistribution = 0;
-	if(nFlags & F64_OPEN_NEW)
-		dwCreationDistribution |= CREATE_ALWAYS;
-	if(nFlags & F64_OPEN_EXISTING)
-		dwCreationDistribution |= OPEN_EXISTING;
-	if((nFlags & (F64_OPEN_NEW|F64_OPEN_EXISTING)) == (F64_OPEN_NEW|F64_OPEN_EXISTING))
-		dwCreationDistribution = OPEN_ALWAYS;
-
-	ASSERT(dwCreationDistribution != 0);
-
-	HANDLE hFile = ::CreateFile(szPath,	
-								dwDesiredAccess,
-								dwShareMode,
-								NULL,	//lpSecurityAttributes,
-								dwCreationDistribution,	
-								0,		//dwFlagsAndAttributes,
-								NULL);	//hTemplateFile);
-	
-	bool bSuccess = false;
-	if (hFile != INVALID_HANDLE_VALUE)
-	{
-		bSuccess = true;
-		m_hFile = hFile;
-	}
-	
-	return bSuccess;
-#else
 	int nOpenFlags = 0;
 	if(nFlags & F64_READ)
 		nOpenFlags |= O_RDONLY;
@@ -89,32 +57,57 @@ bool File64::Open(const char *szPath, unsigned long nFlags)
  #ifdef O_LARGEFILE
 	nOpenFlags |= O_LARGEFILE;
  #endif
-	
-	m_nFile	= open(szPath, nOpenFlags, 0600);	//default permissions are set to 0600
-	return (m_nFile != -1);
+ #ifdef _WIN32
+	nOpenFlags |= _O_BINARY;
+ #endif
+
+	int pmode = 0600; //default permissions are set to 0600
+#ifdef _WIN32
+	pmode = _S_IREAD | _S_IWRITE;
 #endif
+
+//"The foremost problem is GTK+ and GLib APIs that take or return C file
+//descriptors (or FILE pointers). If your code uses a different C
+//runtime, the int returned by g_open() or the FILE* returned by
+//g_fopen() won't mean anything to your code"
+#ifdef G_OS_WIN32
+  if (G_WIN32_HAVE_WIDECHAR_API ())
+    {
+      wchar_t *wfilename = (wchar_t *)g_utf8_to_utf16 (szPath, -1, NULL, NULL, NULL);
+      m_nFile	= _wopen (wfilename, nOpenFlags, pmode);
+      int save_errno = errno;
+
+      g_free (wfilename);
+
+      errno = save_errno;
+    }
+  else
+    {    
+      gchar *cp_filename = g_locale_from_utf8 (szPath, -1, NULL, NULL, NULL);
+      m_nFile	= open (cp_filename, nOpenFlags, pmode);
+      int save_errno = errno;
+
+      g_free (cp_filename);
+
+      errno = save_errno;
+    }
+#else
+	m_nFile	= open(szPath, nOpenFlags, pmode);	
+#endif
+
+	return (m_nFile != -1);
 }
 
 void File64::Close()
 {
-#ifdef _WIN32
-	if (IsOpen())
-		::CloseHandle(m_hFile);
-	m_hFile = INVALID_HANDLE_VALUE;
-#else
 	if(IsOpen())
 		close(m_nFile);
 	m_nFile = -1;
-#endif
 }
 
 bool File64::IsOpen() const
 {
-#ifdef _WIN32
-	return (m_hFile != INVALID_HANDLE_VALUE); 
-#else
 	return (m_nFile != -1);
-#endif
 }
 
 void File64::Flush()
@@ -122,7 +115,7 @@ void File64::Flush()
 	ASSERT(IsOpen());
 
 #ifdef _WIN32	
-	::FlushFileBuffers(m_hFile);
+	//::FlushFileBuffers(m_hFile);
 #else
 	fsync(m_nFile);
 #endif
@@ -132,31 +125,13 @@ int	 File64::Read(char *szBuffer, int nLen)
 {
 	ASSERT(IsOpen());
 
-#ifdef _WIN32
-	DWORD dwRead;
-	if (!::ReadFile(m_hFile, (void *)szBuffer, nLen, &dwRead, NULL))
-		return -1;
-
-	return dwRead;
-#else
 	return read(m_nFile, (void *)szBuffer, nLen);
-#endif
 }
 
 int	File64::Write(const char *szBuffer, int nLen)
 {
 	ASSERT(IsOpen());
 
-#ifdef _WIN32
-	unsigned long nWritten;
-	if (!::WriteFile(m_hFile, (void *)szBuffer, nLen, &nWritten, NULL))
-		return -1;
-
-	if (nWritten != (unsigned int)nLen)
-		return -1;
-
-	return nWritten;
-#else
 	int nWritten;
 
 	nWritten = write(m_nFile, szBuffer, nLen);
@@ -164,64 +139,27 @@ int	File64::Write(const char *szBuffer, int nLen)
 		nWritten = write(m_nFile, szBuffer, nLen);	// retry if interrupted
 
 	return nWritten;
-#endif
 }
 
 INT64 File64::Seek(INT64 offset)
 {
 	ASSERT(IsOpen());
 
-#ifdef _WIN32
-	LARGE_INTEGER li;
-	li.QuadPart = offset;
-	li.LowPart  = ::SetFilePointer(m_hFile, li.LowPart, &li.HighPart, FILE_BEGIN);
-
-	DWORD dw = GetLastError();
-	if ((li.LowPart == 0xFFFFFFFF) && (dw != NO_ERROR))
-		return -1;
-
-	return li.QuadPart;
-#else
 	return lseek(m_nFile, offset, SEEK_SET);
-#endif
 }
 
 INT64 File64::SeekEnd()
 {
 	ASSERT(IsOpen());
 
-#ifdef _WIN32
-	LARGE_INTEGER li;
-	li.QuadPart = 0;
-	li.LowPart  = ::SetFilePointer(m_hFile, li.LowPart, &li.HighPart, FILE_END);
-
-	DWORD dw = GetLastError();
-	if ((li.LowPart == 0xFFFFFFFF) && (dw != NO_ERROR))
-		return -1;
-
-	return li.QuadPart;
-#else
 	return lseek(m_nFile, 0, SEEK_END);
-#endif
 }
 
 INT64 File64::GetPosition() const
 {
 	ASSERT(IsOpen());
 
-#ifdef _WIN32
-	LARGE_INTEGER li;
-
-	li.HighPart = 0;
-	li.LowPart  = ::SetFilePointer(m_hFile, 0, (LONG*)&li.HighPart, FILE_CURRENT);
-	if (li.LowPart  == 0xFFFFFFFF)
-		return -1;
-
-	return li.QuadPart;
-#else
-
 	return lseek(m_nFile, 0, SEEK_CUR);	//just return pos
-#endif
 }
 
 int File64::ReadLine(char *szBuffer, int nLen)
