@@ -50,11 +50,7 @@ public:
 	INT64 SeekEnd();
 
 protected:
-#ifdef _WIN32
-	HANDLE  m_hFile;
-#else
 	int m_nFile;	
-#endif
 };
 
 #endif // FILE64_H__
