@@ -242,3 +242,13 @@ void EnsureExtension(std::string &strPath, const char *szExt)
 	strPath += szExt;	//append new one
 }
 
+
+std::string GetFileExt(const char *szPath)
+{
+	std::string strPath(szPath);
+	size_t nPos = strPath.find_last_of(".");
+	if(nPos != std::string::npos)
+		return strPath.substr(nPos);	//strip old ext
+
+	return "";
+}
