@@ -21,6 +21,9 @@ public:
 	virtual int Save(const char *szFile, NoteDocument &doc);
 	virtual int GetFormat(){ return FORMAT_EXECUTABLE; };
 
+	std::string m_strGeneratorApp;	// name of app read from the document
+	std::string m_strGeneratorVer;	// version of app read from the document
+
 protected:
 	//document parse info
 	int m_nCurParentID;
