@@ -8,7 +8,9 @@
 
 #if _MSC_VER > 1000
   #pragma warning(disable: 4786)
+  #pragma warning(disable: 4996)	//ignore deprecated functions warning
 #endif
+
 #include "FormatIOHtml.h"
 #include "IOLayerFile64.h"
 #include "LinkInfo.h"
@@ -18,13 +20,16 @@
 #include "IniFile.h"
 #include "Base64.h"
 
+#include "../config.h"
 #include "../support.h"	//TOFIX dependent on non-core
 
 #ifndef _WIN32
  #include <strings.h>	//strcasecmp
 #else
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
  #define strcasecmp stricmp
 #endif
@@ -46,6 +51,8 @@ static bool g_bUseTOC = false;
 extern IniFile g_objIni;
 extern bool g_bExportDataAsSeparateFile;
 
+static std::string g_strTabReplacement;
+
 int InternalIcon_Name2Index(const char *szName);
 const char *InternalIcon_Index2Name(int nIndex);
 std::string GetParentDir(const char *szPath);
@@ -76,6 +83,7 @@ FormatIO_HTML::FormatIO_HTML()
 	m_nLinkIdx = -1;
 	m_nExportImgCount = 0;
 	m_nExportAttCount = 0;
+	m_nPicCountInCurNode = 0;
 	m_bExportOverwrite = false;
 	m_bBRTagUsed = false;
 }
@@ -171,18 +179,27 @@ void FormatIO_HTML::WriteDocContents(IOProcess &objFile)
 			objFile.WriteString("<!DOCTYPE NoteCase-File>\n");
 	
 		//calc and write selected node "index"
-		char szLastNote[100];
-		snprintf(szLastNote, sizeof(szLastNote), "<!--LastNote:%d-->\n", m_pDoc->m_nCurrentNode);
-		objFile.WriteString(szLastNote);
+		if(m_pDoc->m_nCurrentNode >= 0){
+			char szLastNote[100];
+			snprintf(szLastNote, sizeof(szLastNote), "<!--LastNote:%d-->\n", m_pDoc->m_nCurrentNode);
+			objFile.WriteString(szLastNote);
+		}
 	}
 	else
 		objFile.WriteString("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">");
 
 	//write inital HTML tags including HEAD section
 	objFile.WriteString("<HTML>\n<HEAD>\n");
+
 	//define used charset
 	objFile.WriteString("<meta content=\"text/html;charset=UTF-8\" http-equiv=\"Content-Type\">\n");
 
+	//define application that generated this document
+	objFile.WriteString("<meta name=\"generator\" content=\"");
+	objFile.WriteString(APP_NAME_STR);
+	objFile.WriteString(" ");
+	objFile.WriteString(APP_VER_STR);
+	objFile.WriteString("\">\n"); //<meta name="generator" content="WordPress 2.0.5" />
 	
 	if(m_bHtmlExport)
 	{
@@ -215,6 +232,13 @@ void FormatIO_HTML::WriteDocContents(IOProcess &objFile)
 	g_objIni.GetValue("Export", "TOCDepth", g_nTocDepth, 1);
 	g_objIni.GetValue("Export", "ConvertImageLinks", g_bConvertImageLinks);
 
+	//calculate tab replacement string
+	int nTabSize;
+	g_objIni.GetValue("Export", "TabSize", nTabSize, 3);
+	g_strTabReplacement = "";
+	for(int i=0; i<nTabSize; i++)
+		g_strTabReplacement += "&nbsp;";
+
 	if(m_bHtmlExport)
 	{
 		if(m_pDoc->m_strCustomCSS.size() > 0)
@@ -250,6 +274,10 @@ void FormatIO_HTML::WriteDocContents(IOProcess &objFile)
 
 	objFile.WriteString("</HEAD>\n<BODY>\n");
 
+	//write read-only fiag (this one is global - per document)
+	if(m_pDoc->IsReadOnly())
+		objFile.WriteString("<!--property:read_only-->\n");
+
 	DocumentIterator it(*m_pDoc);
 
 	//write TOC
@@ -282,6 +310,7 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 		m_pDoc->NodeInsert(nPID, -1);
 		m_nCurNodeIdx = m_pDoc->GetNodeCount()-1;
 		m_nDocParseState = DOC_PARSE_EXPECT_TITLE;
+		m_nPicCountInCurNode = 0;
 		ASSERT(m_nCurNodeIdx >= 0);
 	}
 	else if(0 == strcasecmp(szTag, "DD"))	//DD indicates new node text coming
@@ -421,12 +450,14 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 
 				PixInfo info;
 				info.bUsePNG = bUsePNG;
-				info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1) + nCnt;
+				info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1) + nCnt;
 				info.pixbuf  = pixbuf;
 				info.strName = strTitle;
 				m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstPictures.push_back(info);
 			
 				delete [] szDecode;
+
+				m_nPicCountInCurNode ++;
 			}
 		}
 	}
@@ -437,7 +468,7 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_BOLD_BEGIN;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 
 			//TRACE("BOLD begin at %d for node:%d\n", info.nOffset, m_nCurNodeIdx);
@@ -450,7 +481,7 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_ITALIC_BEGIN;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 		}
 	}
@@ -461,7 +492,7 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_UNDERLINE_BEGIN;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 		}
 	}
@@ -472,7 +503,7 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_STRIKE_BEGIN;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 
 			//TRACE("STRIKE begin at %d for node:%d\n", info.nOffset, m_nCurNodeIdx);
@@ -501,12 +532,88 @@ void FormatIO_HTML::OnTagBegin(const char *szTag, const char *szParams)
 				FmtInfo info;
 				info.color = color;
 				info.nFormatTag = FMT_TXT_COLOR_BEGIN;
-				info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+				info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 				m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 			}
 		}
 	}
-	
+	else if(0 == strcasecmp(szTag, "meta"))	//header tag, defines generator app
+	{
+		//extract generator info (application name and version)
+		const char *szGeneratorPos = strstr(szParams, "name=\"generator\"");
+		if(szGeneratorPos)
+		{
+			szGeneratorPos += strlen("name=\"generator\"");
+			szGeneratorPos = strstr(szGeneratorPos, "content=\"");
+			if(szGeneratorPos)
+			{
+				szGeneratorPos += strlen("content=\""); //point to the field data
+				const char *szEnd = strstr(szGeneratorPos, "\"");
+				if(szEnd){
+					std::string strTitle;
+					strTitle.assign(szGeneratorPos, szEnd-szGeneratorPos);
+
+					std::string::size_type nPos = strTitle.find_last_of(" ");
+					if(nPos != std::string::npos){
+						m_strGeneratorVer = strTitle.substr(nPos+1);
+						m_strGeneratorApp = strTitle.substr(0, nPos);
+					}
+				}
+			}
+		}
+	}
+	else if(0 == strcasecmp(szTag, "span"))	//font background color tag begins
+	{
+		ASSERT(m_nCurNodeIdx >= 0);
+		if(m_nCurNodeIdx >= 0)
+		{
+			const char *szColor = strstr(szParams, "style=\"background-color:");
+			if(szColor)
+			{
+				szColor += strlen("style=\"background-color:");
+				std::string strData = szColor;
+
+				const char *szEnd = strstr(szColor, "\"");
+				if(szEnd){
+					int nEndOffset = szEnd-szColor;
+					strData.erase(nEndOffset);
+				}
+
+				GdkColor color;
+				gdk_color_parse (strData.c_str(), &color);
+
+				FmtInfo info;
+				info.color = color;
+				info.nFormatTag = FMT_TXT_BKG_COLOR_BEGIN;
+				info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+				m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
+			}
+			else{
+				//new tag for text color using span
+				szColor = strstr(szParams, "style=\"color:");
+				if(szColor)
+				{
+					szColor += strlen("style=\"color:");
+					std::string strData = szColor;
+
+					const char *szEnd = strstr(szColor, "\"");
+					if(szEnd){
+						int nEndOffset = szEnd-szColor;
+						strData.erase(nEndOffset);
+					}
+
+					GdkColor color;
+					gdk_color_parse (strData.c_str(), &color);
+
+					FmtInfo info;
+					info.color = color;
+					info.nFormatTag = FMT_TXT_COLOR_BEGIN;
+					info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+					m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
+				}
+			}
+		}
+	}
 }
 
 void FormatIO_HTML::OnTagEnd(const char *szTag)	
@@ -608,7 +715,7 @@ void FormatIO_HTML::OnTagEnd(const char *szTag)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_BOLD_END;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 
 			//TRACE("BOLD end at %d for node:%d\n", info.nOffset, m_nCurNodeIdx);
@@ -621,7 +728,7 @@ void FormatIO_HTML::OnTagEnd(const char *szTag)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_ITALIC_END;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 		}
 	}
@@ -632,7 +739,7 @@ void FormatIO_HTML::OnTagEnd(const char *szTag)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_UNDERLINE_END;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 		}
 	}
@@ -643,7 +750,7 @@ void FormatIO_HTML::OnTagEnd(const char *szTag)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_STRIKE_END;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 
 			//TRACE("STRIKE end at %d for node:%d\n", info.nOffset, m_nCurNodeIdx);
@@ -656,10 +763,40 @@ void FormatIO_HTML::OnTagEnd(const char *szTag)
 		{
 			FmtInfo info;
 			info.nFormatTag = FMT_TXT_COLOR_END;
-			info.nOffset = g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+			info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
 			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
 		}
 	}
+	else if(0 == strcasecmp(szTag, "span"))	//font bkg color tag ends
+	{
+		ASSERT(m_nCurNodeIdx >= 0);
+		if(m_nCurNodeIdx >= 0)
+		{
+			//find out what of the two span based tags are we closing
+			FmtInfoList &list = m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt;
+			int nCnt = list.size()-1;
+
+			int nPos1 = list.FindOppositeTag(FMT_TXT_BKG_COLOR_END, nCnt, false, true);
+			int nPos2 = list.FindOppositeTag(FMT_TXT_COLOR_END, nCnt, false, true);
+			
+			if(nPos1 > nPos2)
+			{
+				FmtInfo info;
+				info.nFormatTag = FMT_TXT_BKG_COLOR_END;
+				info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+				info.color = list[nPos1].color;
+				m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
+			}
+			else if(nPos2 >= 0)
+			{
+				FmtInfo info;
+				info.nFormatTag = FMT_TXT_COLOR_END;
+				info.nOffset = m_nPicCountInCurNode + g_utf8_strlen(m_pDoc->GetNodeByIdx(m_nCurNodeIdx).GetText().c_str(), -1);
+				info.color = list[nPos2].color;
+				m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_lstTxtFmt.push_back(info);
+			}
+		}
+	}
 }
 
 void FormatIO_HTML::OnComment(const char *szText)
@@ -747,6 +884,18 @@ void FormatIO_HTML::OnComment(const char *szText)
 		std::string strDate = szText + strlen("property:finished=");
 		m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_bFinished = (atoi(strDate.c_str()) != 0);
 	}
+	else if (0 == strncmp(szText, "property:read_only", strlen("property:read_only")))
+	{
+		if(m_nCurNodeIdx < 0)
+			m_pDoc->SetReadOnly(true);
+	}
+	else if (0 == strncmp(szText, "property:expanded", strlen("property:expanded")))
+	{
+		ASSERT(m_nCurNodeIdx >= 0);
+
+		if(m_nCurNodeIdx >= 0)
+			m_pDoc->GetNodeByIdx(m_nCurNodeIdx).m_bExpanded = true;
+	}
 }
 
 void FormatIO_HTML::OnText(const char *szText)
@@ -918,18 +1067,35 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 		// if the node is a link (source)
 		bool bLink = node.m_objLink.IsValid();
 		if(bLink){
-			char szBuffer[20];
 			std::string strLink("<A href=\"");
-			if(node.m_objLink.m_nTargetNodeID >= 0){
-				//internal link is stored as recursive node index
+
+			//check if internal link became invalid (exporting single node or similar)
+			bool bLinkOK = true;
+			if( node.m_objLink.m_strTargetURL.empty() &&
+				node.m_objLink.m_nTargetNodeID >= 0)
+			{
 				int nIdx = it.GetDocument().GetIdxFromID(node.m_objLink.m_nTargetNodeID);
-				sprintf(szBuffer, "#%d", it.NodeIdx2RecursiveIdx(nIdx));
-				strLink += szBuffer;
+				if(nIdx < 0)
+					bLinkOK = false;
 			}
-			else{
-				std::string strLinkURI = node.m_objLink.m_strTargetURL;
-				HTMLParser::EscapeURI(strLinkURI);
-				strLink += strLinkURI;
+			if(m_bHtmlExport && !bLinkOK)
+			{
+				strLink += "javascript:;\" onclick=\"LinkWarning()";
+			}
+			else
+			{
+				char szBuffer[20];
+				if(node.m_objLink.m_nTargetNodeID >= 0){
+					//internal link is stored as recursive node index
+					int nIdx = it.GetDocument().GetIdxFromID(node.m_objLink.m_nTargetNodeID);
+					sprintf(szBuffer, "#%d", it.NodeIdx2RecursiveIdx(nIdx));
+					strLink += szBuffer;
+				}
+				else{
+					std::string strLinkURI = node.m_objLink.m_strTargetURL;
+					HTMLParser::EscapeURI(strLinkURI);
+					strLink += strLinkURI;
+				}
 			}
 
 			strLink += "\">";
@@ -993,10 +1159,15 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 			file.WriteString(szBuffer);
 			file.WriteString("-->\n");
 		
-			snprintf(szBuffer, sizeof(szBuffer), "%d", node.m_bFinished);
-			file.WriteString("<!--property:finished=");
-			file.WriteString(szBuffer);
-			file.WriteString("-->\n");
+			if(node.m_bFinished){	// save only when set (TOFIX can remove =1 in the future)
+				snprintf(szBuffer, sizeof(szBuffer), "%d", node.m_bFinished);
+				file.WriteString("<!--property:finished=");
+				file.WriteString(szBuffer);
+				file.WriteString("-->\n");
+			}
+
+			if(node.m_bExpanded)	// save only when set
+				file.WriteString("<!--property:expanded-->\n");
 		}
 
 		//
@@ -1053,11 +1224,11 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 
 						int nPos = 0;
 						int nChunkSize = 2048;
-						int nSaveStep  = min(nChunkSize, nSize-nPos);
 						int nLoopCnt = 0;
 						size_t nWritten = 0;
 
 						while(nPos < nSize-1){
+							int nSaveStep  = min(nChunkSize, nSize-nPos);
 							nWritten = fwrite(szData+nPos, 1, nSaveStep, pOut);
 							nPos += nSaveStep;
 							nLoopCnt++;
@@ -1205,20 +1376,36 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 					}
 					if(!bSkipLink)
 					{
-						//link segment
-						char szBuffer[20];
-						std::string strLink;
-						strLink += "<A href=\"";
-						if(link.m_nTargetNodeID >= 0){
-							//internal link is stored as recursive node index
+						std::string strLink = "<A href=\"";
+
+						//check if internal link became invalid (exporting single node or similar)
+						bool bLinkOK = true;
+						if( link.m_strTargetURL.empty() &&
+							link.m_nTargetNodeID >= 0)
+						{
 							int nIdx = it.GetDocument().GetIdxFromID(link.m_nTargetNodeID);
-							sprintf(szBuffer, "#%d", it.NodeIdx2RecursiveIdx(nIdx));
-							strLink += szBuffer;
+							if(nIdx < 0)
+								bLinkOK = false;
 						}
-						else{
-							std::string strLinkURI = link.m_strTargetURL;
-							HTMLParser::EscapeURI(strLinkURI);
-							strLink += strLinkURI;
+						if(m_bHtmlExport && !bLinkOK)
+						{
+							strLink += "javascript:;\" onclick=\"LinkWarning()";
+						}
+						else
+						{
+							//link segment
+							char szBuffer[20];
+							if(link.m_nTargetNodeID >= 0){
+								//internal link is stored as recursive node index
+								int nIdx = it.GetDocument().GetIdxFromID(link.m_nTargetNodeID);
+								sprintf(szBuffer, "#%d", it.NodeIdx2RecursiveIdx(nIdx));
+								strLink += szBuffer;
+							}
+							else{
+								std::string strLinkURI = link.m_strTargetURL;
+								HTMLParser::EscapeURI(strLinkURI);
+								strLink += strLinkURI;
+							}
 						}
 						strLink += "\">";
 
@@ -1551,11 +1738,20 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 						strFmtData += "</s>";
 					else if(FMT_TXT_COLOR_BEGIN == fmt.nFormatTag){
 						char szBuffer[200];
-						sprintf(szBuffer, "<font color=\"#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
+						//sprintf(szBuffer, "<font color=\"#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
+						sprintf(szBuffer, "<span style=\"color:#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
 						strFmtData += szBuffer;
 					}
 					else if(FMT_TXT_COLOR_END == fmt.nFormatTag)
-						strFmtData += "</font>";
+						strFmtData += "</span>";
+					else if (FMT_TXT_BKG_COLOR_BEGIN == fmt.nFormatTag){
+						char szBuffer[200];
+						sprintf(szBuffer, "<span style=\"background-color:#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
+						strFmtData += szBuffer;
+					}
+					else if (FMT_TXT_BKG_COLOR_END == fmt.nFormatTag)
+						strFmtData += "</span>";
+
 					//TOFIX more tags
 
 					tSegment info;
@@ -1595,12 +1791,19 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 						strFmtData += "</s>";
 					else if(FMT_TXT_COLOR_BEGIN == fmt.nFormatTag){
 						char szBuffer[200];
-						sprintf(szBuffer, "<font color=\"#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
+						//sprintf(szBuffer, "<font color=\"#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
+						sprintf(szBuffer, "<span style=\"color:#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
 						strFmtData += szBuffer;
 					}
 					else if(FMT_TXT_COLOR_END == fmt.nFormatTag)
-						strFmtData += "</font>";
-
+						strFmtData += "</span>";
+					else if (FMT_TXT_BKG_COLOR_BEGIN == fmt.nFormatTag){
+						char szBuffer[200];
+						sprintf(szBuffer, "<span style=\"background-color:#%02x%02x%02x\">", fmt.color.red/256, fmt.color.green/256, fmt.color.blue/256);
+						strFmtData += szBuffer;
+					}
+					else if (FMT_TXT_BKG_COLOR_END == fmt.nFormatTag)
+						strFmtData += "</span>";
 					//TOFIX more tags
 
 					tSegment info;
@@ -1627,6 +1830,12 @@ void FormatIO_HTML::WriteLevel(DocumentIterator &it, IOProcess &file, int nParen
 				replaceall(strTmp, "\r\n", "\r");
 				replaceall(strTmp, "\n", "\r");
 				replaceall(strTmp, "\r", "<BR>\n");
+
+				if(bHtmlExport){
+					//convert spaces to non-breaking space
+					replaceall(strTmp, " ", "&nbsp;");
+					replaceall(strTmp, "\t", g_strTabReplacement.c_str());
+				}
 			}
 			strData += strTmp;
 		}
@@ -1669,6 +1878,7 @@ void FormatIO_HTML::WriteTOC(DocumentIterator &it, IOProcess &file, int nParentI
 	int nSiblingIdx = 0;
 
 	if(0 == nDepth){
+		file.WriteString("<hr></hr>");	// horiz. line
 		file.WriteString("<b>");
 		file.WriteString(_("Contents"));
 		file.WriteString("</b>\n<br>\n");
@@ -1712,6 +1922,10 @@ void FormatIO_HTML::WriteTOC(DocumentIterator &it, IOProcess &file, int nParentI
 
 	//if(nDepth > 0)
 		file.WriteString("</ul>\n");
+
+	if(0 == nDepth){
+		file.WriteString("<hr></hr>");	// horiz. line
+	}
 }
 
 void replaceall(std::string &strData, const char *szFind, const char *szReplace)
