@@ -20,7 +20,9 @@
 #include "debug.h"
 #ifdef _WIN32
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
 #endif
 
