@@ -6,6 +6,9 @@
 // File: Class that stores/manipulates NoteCase document contents 
 ////////////////////////////////////////////////////////////////////////////
 
+#if _MSC_VER > 1000
+  #pragma warning(disable: 4996)	//ignore deprecated functions warning
+#endif
 #include "SHA1.h"	// must be the top one, because its defines can cause linker error
 #include "NoteDocument.h"
 #include "FormatIOHtml.h"
@@ -16,6 +19,7 @@
 #include "FormatIOMMLX.h"
 #include "FormatIOExecutable.h"
 #include "DocumentIterator.h"
+#include "FilePath.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -24,6 +28,13 @@
 #include "../gui/ProgressDlg.h"
 #include "../support.h"
 
+#include <gtk/gtk.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+
+int gtkMessageBox(const char *szText, int nButtons = GTK_BUTTONS_OK, int nIcon = GTK_MESSAGE_INFO); //TOFIX temp
+int cmpver(const char* szVer1, const char* szVer2);
+
 extern GtkWidget *window1;
 std::string GetAppPath();
 
@@ -44,9 +55,13 @@ NoteDocument::NoteDocument()
 	m_nIDGenerator = 0;	//first valid node ID
 	m_nCurrentNode = -1;
 	m_pfnPassCallback = NULL;
+	m_pfnFmtCallback = NULL;
 	m_nNodesCount = 0;
 	m_nActiveNodeIdx = -1;
 	m_nEmbedCSS = -1;
+	m_bModified = false;
+	m_bReadOnly = false;
+	m_bModifiedSinceAutosave = false;
 }
 
 NoteDocument::~NoteDocument()
@@ -69,12 +84,14 @@ void NoteDocument::operator = (const NoteDocument &doc)
 	m_pfnPassCallback = doc.m_pfnPassCallback;
 	m_lstNodes        = doc.m_lstNodes;
 	m_bModified       = doc.m_bModified;	
+	m_bModifiedSinceAutosave       = doc.m_bModifiedSinceAutosave;	
 	m_strPath         = doc.m_strPath;
 	m_strPassword     = doc.m_strPassword;
 	m_nNodesCount     = doc.m_nNodesCount;
 	m_pfnPassCallback = doc.m_pfnPassCallback;
 	m_mapIdIsTarget   = doc.m_mapIdIsTarget;
 	m_mapId2Idx       = doc.m_mapId2Idx;
+	m_bReadOnly		  = doc.m_bReadOnly;
 
 #ifdef _DEBUG
 	AssertValid();
@@ -90,7 +107,8 @@ void NoteDocument::Close()
 	m_pObjLoader = NULL;	
 	
 	m_strPath = "";	//forget path
-	m_bModified = false;
+	SetModified(false);
+	m_bReadOnly = false;
 }
 
 void NoteDocument::Clear()
@@ -151,13 +169,17 @@ int NoteDocument::Load(const char *szPath, const char *szPassword, int nFormat)
 #endif
 
 	//check if file exists
-	if(0 != access(szPath, 0))
+	if(0 != g_access(szPath, 0))
 		return DOC_LOAD_NOT_FOUND;
 
+	//test if file readable
+	if(0 != g_access(szPath, 04))
+		return DOC_LOAD_PERMISSIONS;
+
 	//TOFIX special factory fn.
 	if(FORMAT_NONE == nFormat)
 		nFormat = GetFormatFromFileName(szPath);
-	if(nFormat <= 0)
+	if(nFormat <= 0 && NULL == m_pfnFmtCallback)
 		return DOC_LOAD_UNKNOWN_FORMAT;
 
 	m_strPassword = ""; //forget password
@@ -211,6 +233,10 @@ int NoteDocument::Load(const char *szPath, const char *szPassword, int nFormat)
 	{
 		m_pObjLoader = new FormatIO_MMLX;
 	}
+	else if(FORMAT_TXT == nFormat)
+	{
+		m_pObjLoader = new FormatIO_Txt;
+	}
 	else if(FORMAT_EXECUTABLE == nFormat)
 	{
 		//guess if the default password should be used (grab password hash SHA1)
@@ -291,10 +317,27 @@ int NoteDocument::Load(const char *szPath, const char *szPassword, int nFormat)
 		m_pObjLoader = new FormatIO_Executable;	// load from this .exe image
 	}
 	else
-		return DOC_LOAD_UNKNOWN_FORMAT;
+	{
+		//load unknown format as text ?
+		if(NULL != m_pfnFmtCallback)
+		{
+			int nFmt = m_pfnFmtCallback();
+			if(FORMAT_TXT == nFmt)
+			{
+				m_pObjLoader = new FormatIO_Txt;
+				nFormat = FORMAT_TXT;
+			}
+			else
+				return DOC_LOAD_UNKNOWN_FORMAT;
+		}
+		else
+			return DOC_LOAD_UNKNOWN_FORMAT;
+	}
 
 	//show wait dialog (only if not from cmd line)
+#ifndef _NOKIA_MAEMO	
 	ProgressDlg dlg(200, _("Loading file. Please wait..."), window1);
+#endif
 
 	if(FORMAT_EXECUTABLE != nFormat)
 		m_strPath = szPath;	//store path
@@ -306,15 +349,37 @@ int NoteDocument::Load(const char *szPath, const char *szPassword, int nFormat)
 	{
 		SetModified(false);	//just loaded
 		m_nNodesCount = m_lstNodes.size();	//cache node count
+
+		//check generator application, issue warning if loading document generated
+		//by another application or newer version of this application
+		if( FORMAT_NOTECASE_HTML == nFormat || 
+			FORMAT_EXECUTABLE	 == nFormat ||
+			FORMAT_NOTECASE_HTML_ENC == nFormat)
+		{
+			std::string	strGeneratorApp, strGeneratorVer;
+			
+			if(FORMAT_EXECUTABLE == nFormat){
+				strGeneratorApp = ((FormatIO_Executable *)m_pObjLoader)->m_strGeneratorApp;
+				strGeneratorVer = ((FormatIO_Executable *)m_pObjLoader)->m_strGeneratorVer;
+			}
+			else{
+				strGeneratorApp = ((FormatIO_HTML *)m_pObjLoader)->m_strGeneratorApp;
+				strGeneratorVer = ((FormatIO_HTML *)m_pObjLoader)->m_strGeneratorVer;
+			}
+
+			if( strGeneratorApp.size() > 0 &&
+				(0 != strcmp(strGeneratorApp.c_str(), APP_NAME_STR) ||
+				 0 <  cmpver(strGeneratorVer.c_str(), APP_VER_STR)))
+			{
+				gtkMessageBox(_("Warning:\nThis document was created either by different application,\nor by newer version of this program.\n\nIf you edit this document, you might lose some formatting!"));
+			}
+		}
 	}
 	else{
 		m_strPassword = ""; //forget password
 		m_strPath = "";
 	}
 
-	//destroy progress window
-	dlg.Close();
-
 	return nResult;
 }
 
@@ -332,12 +397,16 @@ int NoteDocument::Save(const char *szPath, bool bRememberPath, int nBackupFiles,
 	if(nFormat <= 0)
 		return DOC_SAVE_UNKNOWN_FORMAT;
 
+	//ensure directory exists
+	std::string strDir = GetParentDir(szPath);
+	EnsureDirExists(strDir);
+
 	// if the target file already exists implement 2 step saving
 	// - first save to a temp file, then overwrite target file with the temp file
 	// - this is good to preserve file data in case Notecase crashes while saving
 
 	bool bTwoStepSave = false;
-	if(0 == access(szPath, 0))
+	if(0 == g_access(szPath, 0))
 		bTwoStepSave = true;
 
 	std::string strTargetPath(szPath);
@@ -348,11 +417,7 @@ int NoteDocument::Save(const char *szPath, bool bRememberPath, int nBackupFiles,
 	{
 		//quick FIX: delete file if it already exists
 		//TOFIX more precise fix would be to test/fix file open flags when saving
-	#ifdef _WIN32
-		::DeleteFile(szPath);
-	#else
-		remove(szPath);
-	#endif
+		g_remove(szPath);
 	}
 
 	if(FORMAT_NOTECASE_HTML == nFormat)
@@ -401,7 +466,7 @@ int NoteDocument::Save(const char *szPath, bool bRememberPath, int nBackupFiles,
 		}
 
 		m_pObjLoader = new FormatIO_EncHTML;
-		
+
 		//set password to the loader
 		((FormatIO_EncHTML *)m_pObjLoader)->SetPassword(m_strPassword.c_str());
 	}
@@ -417,7 +482,9 @@ int NoteDocument::Save(const char *szPath, bool bRememberPath, int nBackupFiles,
 		return DOC_SAVE_UNKNOWN_FORMAT;
 		
 	//show wait dialog (only if not from cmd line)
+#ifndef _NOKIA_MAEMO
 	ProgressDlg dlg(200, _("Saving file. Please wait..."), window1);
+#endif
 
 	if(bRememberPath)
 		m_strPath = szPath;	//store path
@@ -466,46 +533,26 @@ int NoteDocument::Save(const char *szPath, bool bRememberPath, int nBackupFiles,
 				sprintf(szPathBak, "%s.bak%d", szPath, nBakIdx);
 
 				//rename .bak file if already exists
-				#ifdef _WIN32
-					::DeleteFile(szPathBak);
-				#else
-					remove(szPathBak);
-				#endif
+				g_remove(szPathBak);
 
 				//then rename existing file as .bak (to make room for new file)
-			#ifdef _WIN32
-				bool bRes = (FALSE != ::MoveFile(szPath, szPathBak));
-			#else
-				bool bRes = (0 == rename(szPath, szPathBak));
-			#endif
+				bool bRes = (0 == g_rename(szPath, szPathBak));
 				if(!bRes)
 				{
 					//error creating backup file
 					ASSERT(FALSE);
-					#ifdef _WIN32
-						::DeleteFile(szPath);
-					#else
-						remove(szPath);
-					#endif
+					g_remove(szPath);
 				}
 			}
 			else
 			{
 				//quick FIX: delete target file if it already exists
 				//TOFIX more precise fix would be to test/fix file open flags when saving
-			#ifdef _WIN32
-				::DeleteFile(szPath);
-			#else
-				remove(szPath);
-			#endif
+				g_remove(szPath);
 			}
 
 			//move temporary file to target path
-		#ifdef _WIN32
-			bool bRes = (FALSE != ::MoveFile(strTargetPath.c_str(), szPath));
-		#else
-			bool bRes = (0 == rename(strTargetPath.c_str(), szPath));
-		#endif
+			bool bRes = (0 == g_rename(strTargetPath.c_str(), szPath));
 			if(!bRes)
 				return DOC_SAVE_ERR_OVERWRITE; // Failed to owerwrite target file
 		}
@@ -513,11 +560,6 @@ int NoteDocument::Save(const char *szPath, bool bRememberPath, int nBackupFiles,
 		if(bRememberPath)
 			SetModified(false);	//just saved to known location
 	}
-	else
-		m_strPassword = ""; //forget password
-
-	//destroy progress window
-	dlg.Close();
 
 	return nSaveRes;
 }
@@ -540,6 +582,17 @@ bool NoteDocument::IsModified()
 void NoteDocument::SetModified(bool bModified)
 {
 	m_bModified = bModified;
+	m_bModifiedSinceAutosave = bModified;
+}
+
+bool NoteDocument::IsReadOnly()
+{
+	return m_bReadOnly;
+}
+
+void NoteDocument::SetReadOnly(bool bReadOnly)
+{
+	m_bReadOnly = bReadOnly;
 }
 
 int NoteDocument::GetFormat()
@@ -587,7 +640,7 @@ bool NoteDocument::NodeInsert(int nParentID, int nSiblingIdx, bool bRefreshMap)
 
 	m_nNodesCount  ++;
 	m_nIDGenerator ++;
-	m_bModified = true;
+	SetModified(true);
 
 	return true;
 }
@@ -595,7 +648,7 @@ bool NoteDocument::NodeInsert(int nParentID, int nSiblingIdx, bool bRefreshMap)
 bool NoteDocument::NodeRename(int nID, const char *szTitle)
 {
 	//TOFIX not used, unfinished
-	m_bModified = true;
+	SetModified(true);
 	return true;
 }
 
@@ -628,7 +681,7 @@ bool NoteDocument::NodeDelete(int nID)
 	AssertValid();
 #endif
 
-	m_bModified = true;
+	SetModified(true);
 	return true;
 }
 
@@ -741,7 +794,7 @@ bool NoteDocument::NodeMove(int nID, int nNewParentID, int nSiblingIdx)
 	AssertValid();
 #endif
 	
-	m_bModified = true;
+	SetModified(true);
 	return true;
 }
 
@@ -970,6 +1023,8 @@ void NoteDocument::AssertValid()
 		//TEST 4: each ID value must be >= 0
 		ASSERT(nID >= 0);
 
+		ASSERT(GetNodeByIdx(i).m_nSiblingIdx >= 0);
+
 		//TEST 5: each node in the tree must have unique ID value
 		for(int j=0; j<nCnt; j++){
 			if(i != j)
@@ -1069,11 +1124,12 @@ bool NoteDocument::MoveNodeLeft(int nIdx)
 		int nNewSIB = GetNodeByIdx(nIdx2).m_nSiblingIdx + 1;
 
 		//refresh indexes of new siblings
-		unsigned int i;
-		for(i=0; i<GetNodeCount(); i++)
+		int i;
+		int nCnt = GetNodeCount();
+		for(i=0; i<nCnt; i++)
 		{
 			if( GetNodeByIdx(i).m_nParentID == nNewParentID &&
-				GetNodeByIdx(i).m_nSiblingIdx >= nNewSIB) 
+				GetNodeByIdx(i).m_nSiblingIdx >= nNewSIB && i != nIdx) 
 			{
 				GetNodeByIdx(i).m_nSiblingIdx ++;
 			}
@@ -1083,7 +1139,7 @@ bool NoteDocument::MoveNodeLeft(int nIdx)
 		GetNodeByIdx(nIdx).m_nSiblingIdx = nNewSIB;
 		
 		//refresh indexes of previous siblings
-		for(i=0; i<GetNodeCount(); i++)
+		for(i=0; i<nCnt; i++)
 		{
 			if( GetNodeByIdx(i).m_nParentID == nParentID &&
 				GetNodeByIdx(i).m_nSiblingIdx > nSIB) 
@@ -1121,3 +1177,55 @@ int NoteDocument::CalcNodeLevel(int nIdx)
 	}
 	return nLevel;
 }
+
+//proper version string comparison ("1.5.9" < "1.5.10")
+int cmpver(const char* szVer1, const char* szVer2)
+{
+	std::string strVer1(szVer1);
+	std::string strVer2(szVer2);
+
+	while(1){
+		std::string strSegment1, strSegment2;
+
+		//extract segment
+		std::string::size_type nPos = strVer1.find(".");
+		if(nPos != std::string::npos){
+			strSegment1 = strVer1.substr(0, nPos);
+			strVer1 = strVer1.substr(nPos+1);
+		}
+		else{
+			strSegment1 = strVer1;
+			strVer1 = "";
+		}
+
+		//extract segment2
+		nPos = strVer2.find(".");
+		if(nPos != std::string::npos){
+			strSegment2 = strVer2.substr(0, nPos);
+			strVer2 = strVer2.substr(nPos+1);
+		}
+		else{
+			strSegment2 = strVer2;
+			strVer2 = "";
+		}
+
+		//compare
+		int nVer1 = 0;
+		if(strSegment1.size() > 0)
+			nVer1 = atoi(strSegment1.c_str());
+
+		int nVer2 = 0;
+		if(strSegment2.size() > 0)
+			nVer2 = atoi(strSegment2.c_str());
+
+		if(nVer1 > nVer2)
+			return 1;
+		if(nVer1 < nVer2)
+			return -1;
+
+		if(0 == strVer1.size() && 0 == strVer2.size())
+			break;
+	}
+
+	return 0; //must be equal
+}
