@@ -43,6 +43,7 @@
 class FormatIO_Base;
 
 typedef const char *(* PASS_CALLBACK)(const char *, bool);
+typedef int (* FMT_CALLBACK)();
 
 //supported formats
 #define FORMAT_NONE					0
@@ -64,7 +65,8 @@ typedef const char *(* PASS_CALLBACK)(const char *, bool);
 #define DOC_LOAD_WRONG_PASSWORD 3	//invalid password
 #define DOC_LOAD_UNKNOWN_FORMAT 4	//unsuported format
 #define DOC_LOAD_FORMAT_ERROR   5	//parse error
-#define DOC_LOAD_ERROR          6	//other error
+#define DOC_LOAD_PERMISSIONS    6	//no read permissions
+#define DOC_LOAD_ERROR          7	//other error
 
 //define Save operation results
 #define DOC_SAVE_NOT_SUPPORTED -1	//load not imlplemented
@@ -104,13 +106,18 @@ public:
 	bool IsEmpty();
 	bool IsModified();
 	void SetModified(bool bModified);
+	bool IsReadOnly();
+	void SetReadOnly(bool bReadOnly);
 	bool IsEncrypted();
 	const char *GetPassword(){ return m_strPassword.c_str(); }
+	void SetPassword(const char *szPass){ m_strPassword = szPass; }
 
 	void SetPassCallback(PASS_CALLBACK pfn){ m_pfnPassCallback = pfn; }
+	void SetFmtCallback(FMT_CALLBACK pfn){ m_pfnFmtCallback = pfn; }
 
 	int  GetFormat();
 	std::string	 GetPath(){ return m_strPath; };	//if attached to file
+	void ResetPath(){ m_strPath = ""; };
 
 	bool NodeInsert(int nParentID, int nSiblingIdx, bool bRefreshMap = true);
 	bool NodeRename(int nID, const char *szTitle);
@@ -140,6 +147,8 @@ public:
 	std::string m_strCustomCSS;				//custom CSS path written on HTML export
 	std::map<int, int>	m_mapIdIsTarget;	//how many internal links point to some node ID
 
+	bool m_bModifiedSinceAutosave;			//any change since last autosave ?
+
 protected:
 	void NodeDeleteRecursive(int nID);
 
@@ -149,10 +158,12 @@ protected:
 	std::map<int, int>	m_mapId2Idx;		//fast conversion from ID to Idx
 	unsigned int m_nNodesCount;
 	bool m_bModified;	
+	bool m_bReadOnly;
 	int  m_nIDGenerator;
 	std::string	m_strPath;
 	std::string	m_strPassword;
 	PASS_CALLBACK m_pfnPassCallback;
+	FMT_CALLBACK m_pfnFmtCallback;			// ast to choose format for the unknown .ext
 };
 
 #endif // NOTEDOCUMENT_H__
