@@ -31,13 +31,14 @@ typedef struct{
 	int nDataSize;	//cached size
 } AttInfo;
 
+class DocAction;
+
 //define icon types
 #define ICON_NONE			-2
 #define ICON_CUSTOM			-1
 #define ICON_INTERNAL_FIRST 0
 #define ICON_INTERNAL_LAST	5
 
-
 class NoteNode  
 {
 public:
@@ -47,6 +48,7 @@ public:
 
 	void operator =(const NoteNode& other);
 
+	bool IsEmpty();
 	void Clear();
 	void CopyState(const NoteNode& other);
 
@@ -68,7 +70,7 @@ public:
 
 	void OnTxtInsert(int nOffset, int nCharsAdded, const char *szTxt);
 	void OnTxtReplaced(int nStart, int nEnd, const char *szTxt);
-	void OnTxtDelete(int nOffset, int nCharsDeleted);
+	void OnTxtDelete(int nOffset, int nCharsDeleted, DocAction *pDelAction = NULL);
 
 	LinkInfoList m_lstLinks;	// hyperlinks within the text of the node
 	LinkInfo	 m_objLink;		// hyperlink on the node title (invalid if the node title is not a link)
@@ -89,6 +91,7 @@ public:
 	time_t	m_nDateModified;
 
 	bool	m_bFinished;	//finished nodes have "strikethrough" font style
+	bool	m_bExpanded;
 
 protected:
 	std::string m_strTitle;		// node title
