@@ -4,12 +4,15 @@
 // This code is licensed under BSD license.See "license.txt" for more details.
 //
 // File: C++ class implementation of the BLOWFISH encryption algorithm
+//       Important: encode/decode buffer size should be divisible with 8 (blowfish block size)
 ////////////////////////////////////////////////////////////////////////////
 
 // _THE BLOWFISH ENCRYPTION ALGORITHM_
 // by Bruce Schneier
 // Revised code--3/20/94
 // Converted to C++ class 5/96, Jim Conger
+// - fixed deleting array in destructor (Miroslav Rajcic)
+// - fixed DWORD definition (was invalid for 64bit build) (Miroslav Rajcic)
 
 #ifndef BLOWFISH_H_INCLUDED_
 #define BLOWFISH_H_INCLUDED_
@@ -17,7 +20,7 @@
 #define MAXKEYBYTES 	56		// 448 bits max
 #define NPASS           16		// SBox passes
 
-#define DWORD  		unsigned long
+#define DWORD  		unsigned int
 #define WORD  		unsigned short
 #define BYTE  		unsigned char
 
