@@ -23,9 +23,9 @@ void Assert(int exp, const char *fmt, ...);
 #else
 
 #define TRACE             (1) ? (void)0 : Trace
-#define INFO(x)           (0)
-#define ASSERT(x)         (0)
-#define ASSERTMSG(x,msg)  (0)
+#define INFO(x)           (1) ? (void)0 : Trace("","")
+#define ASSERT(x)         (1) ? (void)0 : Assert(true,"")
+#define ASSERTMSG(x,msg)  (1) ? (void)0 : Assert
 
 #endif // _DEBUG
 
