@@ -21,6 +21,7 @@
 #include "MainWnd.h"
 #include "lib/CircularBuffer.h"
 #include "lib/EnumDirectory.h"
+#include "lib/Thread.h"
 #include <signal.h>
 
 #ifdef _WIN32
@@ -32,6 +33,10 @@
  #include <unistd.h>		//access
 #endif
 
+#if defined (__FreeBSD__) || defined (__APPLE__)
+ #include <libgen.h>		//basename
+#endif
+
 #ifdef _WIN32
  #include "_win/SingleInstance.h"
  CSingleInstance g_instance("NoteCase");
@@ -40,6 +45,14 @@
  CSingleInstance g_instance("/NoteCase");
 #endif
 
+#ifdef _NOKIA_MAEMO
+	#include <libosso.h>
+	// The main OSSO context of the application
+	static osso_context_t *g_osso = NULL;
+#endif
+
+const char *get_locale_dir();
+
 GtkWidget *window1;
 extern MRU g_objMRU;
 bool g_bMinimizeToTray = false;
@@ -48,10 +61,30 @@ extern NoteDocument g_doc;
 CircularBuffer g_objCrashLog;
 std::string m_strLogFile;
 IniFile g_objIni;
+unsigned int g_nProcessID = 0;
+GuiLanguage g_lang(PACKAGE, get_locale_dir());
+
+bool g_bInitialLoad = false;
+bool g_bLoadEmbedded = false;
+bool g_bUpdateMRU = false;
+std::string g_strLoadDocumentFile;
+std::string g_strLoadDocumentPass;
+int g_nLoadDocumentNodeID;
 
 int  load_file(const char *filename, const char *szPassword = NULL, int nCurNodeIdx = -1);
 int  load_file_embedded(int nCurNodeIdx = -1);
-void export_document(const NoteDocument &document, int nExportMode, const std::string &strExportFile, const std::string &strCSS, int nEmbedCSS = -1, bool bCheckOverwrite = true, bool bExportLinked = false, int nFormat = FORMAT_NONE, int nExportBranchIdx=-1);
+void export_document(const NoteDocument &document, int nExportMode, const std::string &strExportFile, const std::string &strPostProcessScript, const std::string &strCSS, int nEmbedCSS = -1, bool bCheckOverwrite = true, bool bExportLinked = false, int nFormat = FORMAT_NONE, int nExportBranchIdx=-1);
+
+class OpListLang : public Thread
+{
+protected:
+	virtual void MainMethod();
+};
+
+void OpListLang::MainMethod()
+{
+	g_lang.ListAvailableCatalogs();
+}
 
 #ifdef HAVE_GNOME_VFS
  #include <libgnomevfs/gnome-vfs.h>
@@ -96,24 +129,43 @@ std::string GetHelpFile()
 	    0 != access(strHelp.c_str(), 0)) 
 	{
 		//Win32 and Linux debug version
-		strHelp = GetAppPath();
-		strHelp = GetParentDir(strHelp.c_str());
-		strHelp += "help.ncd";
+		std::string strHelp1 = GetParentDir(GetAppPath().c_str());
+		strHelp1 += "help.ncd";
+
+	#ifndef _WIN32
+		if(0 == access(strHelp1.c_str(), 0))
+	#endif
+			strHelp = strHelp1;
 	}
 
 	return strHelp;
 }
 
+const char * get_autosave_dir()
+{
+	static std::string strDir;
+	strDir  = GetHomeDir();
+#ifdef _WIN32
+	strDir += "\\.notecase\\";
+#else
+	strDir += "/.notecase/";
+#endif
+
+	return strDir.c_str();
+}
+
 const char *calculate_autosave_filename()
 {
 	//there exists one autosave file per user
 	static std::string strDir;
-	strDir  = GetHomeDir();
-#ifdef _WIN32
-	strDir += "\\.notecase\\autosave.ncd";
-#else
-	strDir += "/.notecase/autosave.ncd";
-#endif
+	strDir   = get_autosave_dir();
+	strDir  += "autosave";
+
+	char szBuffer[20];
+	sprintf(szBuffer, "_n%d", g_nProcessID);
+
+	strDir += szBuffer;
+	strDir += ".ncd";
 
 	return strDir.c_str();
 }
@@ -123,12 +175,14 @@ const char *calculate_autosave_filename1()
 {
 	//there exists one autosave file per user
 	static std::string strDir;
-	strDir  = GetHomeDir();
-#ifdef _WIN32
-	strDir += "\\.notecase\\autosave.nce";
-#else
-	strDir += "/.notecase/autosave.nce";
-#endif
+	strDir   = get_autosave_dir();
+	strDir  += "autosave";
+
+	char szBuffer[20];
+	sprintf(szBuffer, "_n%d", g_nProcessID);
+
+	strDir += szBuffer;
+	strDir += ".nce";
 
 	return strDir.c_str();
 }
@@ -139,7 +193,7 @@ const char *get_locale_dir()
 #ifdef _WIN32
 	strDir  = GetAppPath();
 	strDir  = GetParentDir(strDir.c_str());
-	strDir += "locale/";
+	strDir += "locale\\";
 #else
 	strDir  = INSTALL_PREFIX;
 	strDir += LOCALE_DIR;
@@ -187,6 +241,7 @@ bool OnCrashLogEnum(const char *szFile, void *data)
 
 void CheckCrashLogFiles()
 {
+#ifdef GENERATE_CRASHLOG
 	//
 	// detect and report crash log files
 	//
@@ -206,9 +261,11 @@ void CheckCrashLogFiles()
 				remove(lstLogs[i].c_str());
 		}
 	}
+#endif	
 }
 
 #ifndef WIN32
+#ifndef _OSX_BUILD
 void lock_process_memory()
 {
     int lock = 0;
@@ -228,8 +285,7 @@ void lock_process_memory()
         fprintf(stderr,"WARNING: Running unprivileged and memory is not locked from paging to disk\n");
 }
 #endif
-
-GuiLanguage g_lang(PACKAGE, get_locale_dir());
+#endif
 
 int main (int argc, char *argv[])
 {
@@ -246,6 +302,7 @@ int main (int argc, char *argv[])
  #endif
 #endif
 
+#ifdef GENERATE_CRASHLOG
 	//install signal handlers to catch crash situations
 	signal(SIGABRT, signal_handler);
 	signal(SIGFPE,  signal_handler);
@@ -264,7 +321,7 @@ int main (int argc, char *argv[])
 	char szBuffer[200];
 	time_t nTime = time(NULL);
 	tm *pCurTm = localtime(&nTime);
-	sprintf(szBuffer, " %02d%02d%02d_%02d%02d%02d", 
+	sprintf(szBuffer, "%02d%02d%02d_%02d%02d%02d", 
 			pCurTm->tm_year%100, 
 			pCurTm->tm_mon + 1, 
 			pCurTm->tm_mday, 
@@ -273,6 +330,12 @@ int main (int argc, char *argv[])
 			pCurTm->tm_sec);
 	m_strLogFile += szBuffer;
 	m_strLogFile += ".log";
+#endif
+
+	//calculate PID
+	g_nProcessID = GetCurrentPID();
+	g_objCrashLog.Printf("INFO: Proces ID = %d\n", g_nProcessID);
+
 
 	//check if this exe has embedded document
 	bool bStandalone = false;
@@ -291,20 +354,40 @@ int main (int argc, char *argv[])
 		fclose(pExe);
 	}
 
+#ifdef _NOKIA_MAEMO
+	// Initialize g_osso
+    g_osso = osso_initialize("notecase", "1.0.0", TRUE, NULL);
+    if(!g_osso)
+    {
+        g_printerr("osso_initialize failed.\n");
+        //return 1;
+    }
+#endif 
+    
 	//check startup options
 	g_objIni.Load(GetIniFile());
 
 #ifndef WIN32
+#ifndef _OSX_BUILD
 	bool bLockMemory = false;
 	g_objIni.GetValue("Security", "LockProcessMemory", bLockMemory);
 	if(bLockMemory)
 		lock_process_memory();
+#endif		
 #endif
 
 	//initialize language system
-	std::string strLocale;
-	g_objIni.GetValue("Display", "Language", strLocale, "");
-	g_lang.Initialize(strLocale.c_str());
+	bool bUseSystemLang;
+	g_objIni.GetValue("Display", "UseSystemLanguage", bUseSystemLang);
+	if(bUseSystemLang)
+		g_lang.InitializeToSystemLang();
+	else{
+		std::string strLocale;
+		g_objIni.GetValue("Display", "Language", strLocale, "");
+		g_lang.Initialize(strLocale.c_str());
+	}
+
+	gdk_threads_enter();
 
 	gtk_init (&argc, &argv);
 
@@ -408,28 +491,40 @@ int main (int argc, char *argv[])
 					if(strLoadFile.size() > 0){
 						int nResult = load_file(strLoadFile.c_str(), strPassword.c_str());
 						if(DOC_LOAD_OK == nResult)
-							export_document(g_doc, bExportBranchOnly? 0 /* branch*/ : 2 /*entire doc*/, strExportFile, strExportCSS, bEmbedCSS, false, bExportLinks, FORMAT_HTML, nExportBranchIdx);
+							export_document(g_doc, bExportBranchOnly? 0 /* branch*/ : 2 /*entire doc*/, strExportFile, "", strExportCSS, bEmbedCSS, false, bExportLinks, FORMAT_HTML, nExportBranchIdx);
 					}
 					return 0;
 				}
 
 				g_objCrashLog.AddMsg("Starting program\n");
-				if(!g_wnd.m_pWidget) g_wnd.Create();
 
 				if(strLoadFile.size() > 0){
 					//gtkMessageBox(strLoadFile.c_str());
-					int nResult = load_file(strLoadFile.c_str(), strPassword.c_str(), nCurNodeIdx);
-					g_objMRU.Change(strLoadFile.c_str(), (DOC_LOAD_NOT_FOUND != nResult));//update MRU unless file not found
+					//int nResult = load_file(strLoadFile.c_str(), strPassword.c_str(), nCurNodeIdx);
+					//g_objMRU.Change(strLoadFile.c_str(), (DOC_LOAD_NOT_FOUND != nResult));//update MRU unless file not found
+
+					g_bInitialLoad = true;
+					g_bUpdateMRU = true;
+					g_strLoadDocumentFile = strLoadFile;
+					g_strLoadDocumentPass = strPassword;
+					g_nLoadDocumentNodeID = nCurNodeIdx;
 				}
+				if(!g_wnd.m_pWidget) g_wnd.Create();
 			}
 			else{
 				//check "reload last file" option
-				if(!g_wnd.m_pWidget) g_wnd.Create();	// this will load MRU
+				g_objMRU.Load();
 
 				bool bLoadLastDoc;
 				if(g_objIni.GetValue("Startup", "ReloadLastDocument", bLoadLastDoc))
-					if(bLoadLastDoc && (g_objMRU.GetSize() > 0))
-						load_file(g_objMRU.GetFile(0), NULL, nCurNodeIdx);
+					if(bLoadLastDoc && (g_objMRU.GetSize() > 0)){
+						g_bInitialLoad = true;
+						g_bUpdateMRU = true;
+						g_strLoadDocumentFile = g_objMRU.GetFile(0);
+						g_nLoadDocumentNodeID = nCurNodeIdx;
+					}
+
+				if(!g_wnd.m_pWidget) g_wnd.Create();	// this will load MRU
 			}
 		}
 	}
@@ -437,14 +532,19 @@ int main (int argc, char *argv[])
 	{
 		//TOFIX read command line for embedded doc also ?
 		//load document from .exe file image
+		g_bInitialLoad = true;
+		g_bLoadEmbedded = true;
+
 		if(!g_wnd.m_pWidget) g_wnd.Create();
-		load_file_embedded(/*nCurNodeIdx*/);
 	}
 
 	if(!g_wnd.m_pWidget) g_wnd.Create();
 
 	CheckCrashLogFiles();
 
+	OpListLang *pOpThread = new OpListLang;
+	pOpThread->Run();
+
 #ifdef HAVE_GNOME_VFS
 	gnome_vfs_init();
 #endif
@@ -455,6 +555,14 @@ int main (int argc, char *argv[])
 	gnome_vfs_shutdown();
 #endif
 
+	gdk_threads_leave();
+
+	delete pOpThread;
+
+#ifdef _NOKIA_MAEMO
+   osso_deinitialize(g_osso);
+#endif
+
 #ifdef _DEBUG
 	//dump last actions
 	//signal_handler(0);
