@@ -15,7 +15,9 @@
 
 #ifdef _WIN32
  #ifndef __MINGW32__
-  #define snprintf _snprintf
+   #ifndef snprintf
+    #define snprintf _snprintf
+   #endif
  #endif
 #endif
 
@@ -49,6 +51,8 @@ void MRU::Clear()
 
 void MRU::Load()
 {
+	m_lstFiles.clear();
+
 	int nMRUCount = 0;
 	if(g_objIni.GetValue("MRU", "Count", nMRUCount, 0))
 	{
@@ -62,7 +66,10 @@ void MRU::Load()
 			char szKey[30]="";
 			snprintf(szKey, sizeof(szKey), "File%d", i+1);
 			g_objIni.GetValue("MRU", szKey, strValue, "");
-			m_lstFiles.push_back(strValue);
+
+			int nIdx = FindEntry(strValue.c_str());	// additionally cleanup the list from duplicates
+			if(nIdx < 0)
+				m_lstFiles.push_back(strValue);
 		}
 	}
 }
@@ -87,17 +94,7 @@ void MRU::Save()
 
 void MRU::Change(const char *szFilename, bool bAdd)
 {
-	//find the file in the list
-	int nIdx = -1;
-	for(unsigned int i=0; i<m_lstFiles.size(); i++)
-	{
-		if(0 == strcmp(m_lstFiles[i].c_str(), szFilename))
-		{
-			nIdx = (int)i;
-			break;
-		}
-	}
-
+	int nIdx = FindEntry(szFilename);
 	//remove item from list
 	if(nIdx >= 0)
 		m_lstFiles.erase(m_lstFiles.begin()+nIdx);
@@ -123,26 +120,47 @@ void MRU::RebuildMenu()
 	GtkWidget *menu5 = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (mru_menuitem), menu5);
 
-	unsigned long i;
-	for(i=0; i<m_lstFiles.size(); i++)
+	int i;
+	int nMax = m_lstFiles.size();
+	for(i=0; i<nMax; i++)
 	{
-		//make sure to convert path to UTF8
-		std::string strFileUtf8;
-		LocaleToUtf8(m_lstFiles[i].c_str(), strFileUtf8);
-
 		//append an item
-		menuitem5 = gtk_menu_item_new_with_label (strFileUtf8.c_str());
+		menuitem5 = gtk_menu_item_new_with_label (m_lstFiles[i].c_str());
 		gtk_widget_show (menuitem5);
 		gtk_container_add (GTK_CONTAINER (menu5), menuitem5);
 		g_signal_connect (GTK_OBJECT (menuitem5),  "activate", G_CALLBACK (on_MRU_activate), (void *)i);
 	}
 
 	//append "Clear list" item
-	if(GetFileCount()>0)
+	if(nMax > 0)
 	{
 		menuitem5 = gtk_menu_item_new_with_label (_(" - Clear list - "));
 		gtk_widget_show (menuitem5);
 		gtk_container_add (GTK_CONTAINER (menu5), menuitem5);
 		g_signal_connect (GTK_OBJECT (menuitem5),  "activate", G_CALLBACK (on_MRU_activate), (void *)i);
 	}
+	else
+	{
+		//append "( empty )" item
+		menuitem5 = gtk_menu_item_new_with_label (_("( empty )"));
+		gtk_widget_show (menuitem5);
+		gtk_container_add (GTK_CONTAINER (menu5), menuitem5);
+		gtk_widget_set_sensitive (menuitem5, FALSE);
+	}
 } 
+
+int MRU::FindEntry(const char* szFilename)
+{
+	//find the file in the list
+	int nIdx = -1;
+	for(unsigned int i=0; i<m_lstFiles.size(); i++)
+	{
+		if(0 == strcmp(m_lstFiles[i].c_str(), szFilename))
+		{
+			nIdx = (int)i;
+			break;
+		}
+	}
+
+	return nIdx;
+}
